--[[
*******************************************************************************

	* 블링크의 전투 메세지 v2.0.x : 사용자 정의 이벤트

	* 기능

		전투메세지에 표현하고 싶은 이벤트가있을경우 추가하게되면 해당 메세지가
		나타납니다.


	* 형식

		- pattern : 채팅창에서 필터링할 문구의 정규식패턴
		- msg : pattern에 의해서 찾은 이벤트에 화면에 뿌릴 메세지 :
			%1=pattern의 첫번째 arg,
			%2=pattern의 두번째 arg,
			...
			%9=pattern의 아홉번째 arg
		- rgb = {r=1.0, g=1.0, b=1.0} 화면에 보여지는 문구의 색상배열 :
			r=RED(0.0~1.0), g=GREEN(0.0~1.0), b=BLUE(0.0~1.0)
		- paneltype : starndard, crit, sticky, x2, fountain, gravity
		- font = 화면에 보여지는 글자폰트를 지정 default = Fonts\K_Damage.TTF
		- fontsize = 화면에 보여지는 문구의 크기 1.0이 기본 : 1.5는 1.5배 크기
		- outline = 글자의 테두리를 지정 "","OUTLINE","THICKOUTLINE" : default = ""
		- soundfile = 이벤트가 발동되면 지정된 사운드파일 실행 : 예) soundfile = "Roar.wav"
		- sound = 이벤트가 발동되면 지정된 사운드 실행 : 예) sound = "AuctionWindowClose"
		- class = 특정 직업만 작동할필요가 있다면 직업명을 입력, 없다면 모든 직업 동일하게 작동함
			class 형식은 단일직업일경우 class="전사"
			여러직업일경우 테이블로 class={"마법사","드루이드","사제"}
		- callback = function(arg1,arg2,..arg9) 이벤트가 발동되면 지정된 함수 실행
			arg1~arg9는 pattern에서 뽑아온 %1~%9 값들
		- classcallback = {table} 직업별로 다르게 실행할 함수가 있다면 실행할 테이블
			{
				["전사"] = callbackfunc1,
				["도적"] = callbackfunc2,
				[".."] = calbackfuncN,
				..
			}
		- showmsg = 0|1|2
			0 : 메세지를 화면에 보이지 않음
			1 or nil : 메세지를 화면에 보임
			2 : callback or classcallback함수의 결과값에 따라 보여줄지 결정함(결과값이 nil이 아니면 보임)
		- isCrit = nil|1 : 문구를 강조할경우

		(옵션)
		- commandkey = (string)커맨드명령어 및 저장데이터 키값
		- command = {table} 해당 메세지의 커맨드명령어 생성테이블
			{
				cmd = {table} 커맨드 명령어들 {"hi","hello"},
				cmdfunc = (function) commandkey에 할당된 값을 0<->1 토글하는 함수,
				helpmsg = 도움말 메세지,
				statusmsg = 현재 설정 상태 메세지
			}
		(옵션2)
		- isStaggered = 1|nil
			1 : 메세지를 좌/우쪽으로 약간 치우치게 출력합니다.
		- scrollFunction = function(panel) 출력메세지의 interval별로 위치정보를 계산하는 함수(자신만의 독특한 움직임을 원한다면 사용하세요)
			return xPos, yPos, fontheight, alpha[, point, relateTo, relatePoint];
			xPos : 현재의 x좌표
			yPos : 현재의 y좌표
			fontheight : 현재 글자크기
			alpha : 현재 투명도(0~1.0)
			(옵션)point : 출력문구의 출력위치 포인트 "CENTER","LEFT","BOTTOM" 등
			(옵션)relateTo : 앵커, 출력문구 출력 기준 "UIParent", "PlayerFrame" 등
			(옵션)relatePoint : 앵커(relateTo)의 기준 "CENTER","LEFT","BOTTOM" 등


*******************************************************************************
]]--





--[[

* 회복관련
자신 또는 타인의 스킬에 의한 체력 회복 메세지 - 지속성
자신의 스킬에 의한 체력 회복 메세지
자신의 스킬에 의한 체력 회복 극대화 메세지
자신의 스킬로 타인을 치유하는 메세지
자신의 스킬로 타인을 치유하는 메세지 - 지속성
자신의 스킬로 타인(NPC)을 치유하는 메세지
자신의 스킬로 타인을 치유하는 극대화 메세지
타인의 스킬에 의한 체력 회복 메세지
타인의 스킬에 의한 체력 회복 극대화 메세지
파워회복 - 지속성
파워회복

* 버프/디버프
버프 얻음
버프 얻음 x ?개
디버프 걸림
디버프 걸림 x ?개

* 환경 피해
숨막힘 데미지
낙하 데미지
화염 데미지
용암 데미지
피로 데미지
독성 데미지

* 피해관련
속성데미지 얻음 - 지속성
NPC에 의한 속성 데미지
NPC에 의한 속성 극대화 데미지
NPC에 의한 평타 데미지
NPC에 의한 치명타 데미지
NPC 스킬에 의한 속성 데미지
NPC 스킬에 의한 속성 극대화 데미지
NPC 스킬에 의한 데미지
NPC 스킬에 의한 치명상 데미지
적플레이어에 의한 속성 데미지
적플레이어에 의한 속성 극대화 데미지
적플레이어에 의한 평타 데미지
적플레이어에 의한 치명타 데미지
적플레이어 스킬에 의한 속성 데미지
적플레이어 스킬에 의한 속성 극대화 데미지
적플레이어 스킬에 의한 데미지
적플레이어 스킬에 의한 치명상 데미지

]]


local BCFG_DEBUFF_LOCALIZE = {
	["Magic"] = "마법",
	["Poison"] = "독",
	["Disease"] = "질병",
	["Curse"] = "저주",
};

-- slash command table
BlinkCombatFeedback_Custom_Commands = {};

-- message table
BlinkCombatFeedback_Custom = {};

function BCF_AddCommand(key,command)
	if ( not key or key=="" ) then
		return false;
	end
	if ( BlinkCombatFeedback_Custom_Commands[key] ) then
		return false;
	end
	BlinkCombatFeedback_Custom_Commands[key] = {
		title = command.title,
		cmd = command.cmd,
		cmdfunc = command.cmdfunc,
		statusmsg = command.statusmsg,
		helpmsg = command.helpmsg,
		default = command.default,
	};
	if ( command.default ) then
		if ( not BCF_GetOption(key) ) then
			BCF_SetOption(key,command.default);
		end
	end
	return true;
end

function BCF_Register(info)
	if ( not info or type(info) ~= 'table' ) then
		return false;
	end
	if ( not info.event or strlen(info.event)==0 ) then
		return false;
	end

	if ( not BlinkCombatFeedback_Custom ) then
		BlinkCombatFeedback_Custom = {};
	end

	if ( info.command and type(info.command) == 'table' ) then
		BCF_AddCommand(info.commandkey,info.command);
		info.command = nil;
	end

	if ( not BlinkCombatFeedback_Custom[info.event] ) then
		BlinkCombatFeedback_Custom[info.event] = {};
		BlinkCombatFeedback_Custom[info.event][1] = info;
		if ( info.event~="VARIABLES_LOADED" and
			info.event~="PLAYER_REGEN_ENABLED" and
			info.event~="PLAYER_REGEN_DISABLED" and
			info.event~="PLAYER_COMBO_POINTS" and
			info.event~="UNIT_HEALTH" and
			info.event~="UNIT_MANA" and
			info.event~="COMBAT_TEXT_UPDATE"
		) then
			local frame = getglobal("BlinkCombatFeedbackFrame");
			frame:RegisterEvent(info.event);
		end
	else
		table.insert(BlinkCombatFeedback_Custom[info.event],info);
	end
	
	return true;
end

-- 오버힐량 측정 함수(섬게이트의 cikandin[페어레이디]님께서 제공해주신 함수)
function BCF_GetOverheal(target,damage)
	local unit = BCF_GetTargetUnit(target);
	if unit then
		if ( UnitHealthMax(unit) == 100 ) then
			return damage;
		end
		local lost = UnitHealthMax(unit)-UnitHealth(unit);
		local overheal = damage - lost;
		if (overheal > 0) then
			damage = lost.." {"..overheal.."}";
		end
	end
	return damage;
end

-- 오버힐에 쓰이는 타겟값 리턴함수(섬게이트의 cikandin[페어레이디]님께서 제공해주신 함수)
function BCF_GetTargetUnit(name)
	local unit;
	if ( name == "player" or name == UnitName("player") ) then
		unit = "player";
	elseif ( UnitExists("target") and UnitName("target") == name) then
		unit = "target";
	elseif ( UnitExists("pet") and UnitName("pet") == name) then
		unit = "pet";
	end
	if not unit then
		for i = 1, GetNumRaidMembers(), 1 do
			if ( UnitName("raid" .. i) and UnitName("raid" .. i) == name ) then
				unit = "raid"..i;
				break;
			end
		end
	end
	if not unit then
		for i = 1, GetNumPartyMembers(), 1 do
			if ( UnitName("party" .. i) and UnitName("party" .. i) == name ) then
				unit = "party"..i;
				break;
			end
		end
	end
	return unit;
end

function BCF_FindSuffix(msg)
	local suffix = "";
	local _,_,block = string.find(msg, "(%d+)의 피해 방어");
	if ( block and tonumber(block) > 0 ) then
		suffix = suffix .. string.format(BLOCK_TRAILER,block);
	end
	local _,_,absorb = string.find(msg, "(%d+) 흡수됨");
	if ( absorb and tonumber(absorb) > 0 ) then
		suffix = suffix .. string.format(ABSORB_TRAILER,absorb);
	end
	local _,_,resist = string.find(msg, "(%d+) 저항함");
	if ( resist and tonumber(resist) > 0 ) then
		suffix = suffix .. string.format(RESIST_TRAILER,resist);
	end
	local _,_,vulnerable = string.find(msg,"(%d+) 약점 보너스");
	if ( vulnerable and tonumber(vulnerable) > 0 ) then
		suffix = suffix .. string.format(VULNERABLE_TRAILER,vulnerable);
	end
	return suffix;
end

function BCF_RegisterDefaultEvents()

	if ( not BCF_Register ) then
		return;
	end

-------------------------------------
-- CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS
-------------------------------------

	-- 스킬에 의한 자신의 체력 회복 메세지(지속적) - 0205
	BCF_Register({
		event = "CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS",
		pattern = "당신은 (.+)|1으로;로; 인해 생명력이 (%d+)만큼 회복되었습니다.",
		showmsg = 0,
		callback = function(event,msg,spell,heal)
			local healing = heal;
			local text;
			if ( BCF_GetOption("SHOWOVERHEAL") == 1 ) then
				healing = BCF_GetOverheal("player",heal);
			end
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				local who,spell2
				if SW_FixLogStrings then
					_,_,who,spell2 = string.find(spell,"(.+) 의 (.+)");
				else
					_,_,who,spell2 = string.find(spell,"(.-)의 (.+)");
				end
				if ( who and spell2 ) then
					if ( who == "재기" and spell2 == "바람" ) then -- 전사 스킬 "재기의 바람" 예외처리
						text = spell..": +"..healing;
					elseif ( who == "나루" and spell2 == "선물" ) then -- 드레나이 종족스킬 "나루의 선물" 예외처리
						text = spell..": +"..healing;
					elseif ( who == "자신" ) then
						text = spell2..": +"..healing;
					else
						text = who..": +"..healing.."("..spell2..")";
					end
				else
					text = spell..": +"..healing;
				end
			else
				text = "+"..healing;
			end
			BCF_DisplayMessage({
				text = text,
				fontheight = BCF_GetOption("FONTSIZE"),
				rgb = BCF_GetColor("HEAL"),
				paneltype = "fountain",
				font = "Fonts\\K_Damage.TTF",
			});
		end,
		commandkey = "SHOWHEAL",
		command = {
			title = "체력 회복 메세지",
			cmd = {"회복","heal"},
			helpmsg = "%s: 체력 회복 수치를 보여줍니다.",
			statusmsg = " 체력 회복 메세지 : %s",
			cmdfunc = function ()
				local r = BCF_GetOption("SHOWHEAL");
				if ( r == 1 ) then
					BCF_SetOption("SHOWHEAL", 0);
				else
					BCF_SetOption("SHOWHEAL", 1);
				end
			end,
			default = 1,
		},
	});

	-- 스킬에 의한 자신의 체력 회복 극대화 메세지(지속적) ? 있는 문구인가?
	BCF_Register({
		event = "CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS",
		pattern = "당신은 (.+)|1이;가; 극대화 효과를 발휘하여 생명력이 (%d+)만큼 회복되었습니다.",
		showmsg = 0,
		callback = function(event,msg,spell,heal)
			local healing = heal;
			local text;
			if ( BCF_GetOption("SHOWOVERHEAL") == 1 ) then
				healing = BCF_GetOverheal("player",heal);
			end
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				local who,spell2
				if SW_FixLogStrings then
					_,_,who,spell2 = string.find(spell,"(.+) 의 (.+)");
				else
					_,_,who,spell2 = string.find(spell,"(.-)의 (.+)");
				end
				if ( who and spell2 ) then
					if ( who == "재기" and spell2 == "바람" ) then -- 전사 스킬 "재기의 바람" 예외처리
						text = spell..": +"..healing;
					elseif ( who == "나루" and spell2 == "선물" ) then -- 드레나이 종족스킬 "나루의 선물" 예외처리
						text = spell..": +"..healing;
					elseif ( who == "자신" ) then
						text = spell2..": +"..healing;
					else
						text = who..": +"..healing.."("..spell2..")";
					end
				else
					text = spell..": +"..healing;
				end
			else
				text = "+"..healing;
			end
			BCF_DisplayMessage({
				text = text,
				fontheight = BCF_GetOption("FONTSIZE") * 1.25,
				rgb = BCF_GetColor("HEAL"),
				paneltype = "crit",
				font = "Fonts\\K_Damage.TTF",
			});
		end,
		commandkey = "SHOWHEAL",
		-- 이 메세지의 옵션은 "자신의 스킬에 의한 자신의 체력 회복 메세지(지속적)"에서 이미 설정했으므로 command를 또 설정할 필요없고 commandkey만 동일하게 지정하면 됨
	});

	-- 스킬에 의한 소환수의 체력 회복 메세지
	BCF_Register({
		event = "CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS",
		pattern = "(.+)|1이;가; 당신의 (.+)|1으로;로; 인해 생명력이 (%d+)만큼 회복되었습니다.",
		showmsg = 0,
		callback = function(event,msg,pet,spell,heal)
			local healing = heal;
			if ( BCF_GetOption("SHOWOVERHEAL") == 1 ) then
				healing = BCF_GetOverheal(pet,heal);
			end
			local text;
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				text = pet..": +"..healing.."("..spell..")";
			else
				text = pet..": +"..healing;
			end
			BCF_DisplayMessage({
				text = text,
				fontheight = BCF_GetOption("FONTSIZE"),
				rgb = BCF_GetColor("HEAL"),
				paneltype = "fountain",
				font = "Fonts\\K_Damage.TTF",
			});
		end,
		commandkey = "SHOWDOTHEAL",
		-- 이 메세지의 옵션은 "자신의 스킬에 의한 자신의 체력 회복 메세지(지속적)"에서 이미 설정했으므로 command를 또 설정할 필요없고 commandkey만 동일하게 지정하면 됨
	});

	-- 버프메세지 : 1개짜리 - 0205
	BCF_Register({
		event = "CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS",
		pattern = "당신은 (.+) 효과를 얻었습니다.$",
		msg = "+(%1)",
		rgb = {r=1.0,g=1.0,b=1.0},
		font = "Fonts\\2002b.TTF",
		outline = "OUTLINE",
		paneltype = "standard",
		commandkey = "SHOWBUFFING",
		command = {
			title = "버프 메세지",
			cmd = {"버프","buffing"},
			helpmsg = "%s: 버프 효과를 얻을때 메세지를 보여줍니다.",
			statusmsg = " 버프 메세지 : %s",
			cmdfunc = function ()
				local r = BCF_GetOption("SHOWBUFFING");
				if ( r == 1 ) then
					BCF_SetOption("SHOWBUFFING", 0);
				else
					BCF_SetOption("SHOWBUFFING", 1);
				end
			end,
		},
	});

	-- 버프메세지 : n개짜리 - 0205
	BCF_Register({
		event = "CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS",
		pattern = "당신은 (.+) 효과를 얻었습니다. %((%d+)%)",
		msg = "+(%1) x %2",
		rgb = {r=1.0,g=1.0,b=1.0},
		font = "Fonts\\2002b.TTF",
		outline = "OUTLINE",
		commandkey = "SHOWBUFFING",
		paneltype = "standard",
	});

	-- 파워회복(틱) 자신의 스킬
	BCF_Register({
		event = "CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS",
		pattern = "당신은 (.+)|1으로;로; 인해 (%d+)의 (.+)|1을;를; 얻었습니다.",
		showmsg = 0,
		callback = function(event,msg,spell,power,ptype)
			local rgb;
			local text;
			if ( BCF_GetOption("SHOWTYPECOLOR") == 1 ) then
				if ( ptype == "분노" ) then
					rgb = BCF_GetColor("GAINRAGE");
				elseif ( ptype == "마나" ) then
					rgb = BCF_GetColor("GAINMANA");
				elseif ( ptype == "기력" ) then
					rgb = BCF_GetColor("GAINENERGY");
				else
					rgb = BCF_GetColor("GAINPOWER");
				end
			else
				rgb = BCF_GetColor("GAINPOWER");
			end
			text = "+"..power.." "..ptype;
			BCF_DisplayMessage({
				text = text,
				rgb = rgb,
				fontsize = BCF_GetOption("FONTSIZE"),
				paneltype = "fountain",
			});
		end,
		commandkey = "SHOWGAINPOWER",
	});

--------------------------------------
-- CHAT_MSG_SPELL_PERIODIC_PARTY_BUFFS
--------------------------------------

	-- 자신의 스킬에 의한 파티원의 체력 회복 메시지
	BCF_Register({
		event = "CHAT_MSG_SPELL_PERIODIC_PARTY_BUFFS",
		pattern = "(.+)|1이;가; 당신의 (.+)|1으로;로; 인해 생명력이 (%d+)만큼 회복되었습니다.",
		showmsg = 0,
		callback = function(event,msg,target,spell,heal)
			local healing = heal;
			if ( BCF_GetOption("SHOWOVERHEAL") == 1 ) then
				healing = BCF_GetOverheal(target,heal);
			end
			local text;
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				text = target..": +"..healing.."("..spell..")";
			else
				text = target..": +"..healing;
			end
			BCF_DisplayMessage({
				text = text,
				fontheight = BCF_GetOption("FONTSIZE"),
				rgb = BCF_GetColor("HEAL"),
				paneltype = "fountain",
				font = "Fonts\\K_Damage.TTF",
			});
		end,
		commandkey = "SHOWDOTHEAL",
		command = {
			title = "지속적인 체력 회복 메세지",
			cmd = {"지속회복","dotheal"},
			helpmsg = "%s: 지속적인 체력 회복 수치를 보여줍니다.",
			statusmsg = " 지속적인 체력 회복 메세지 : %s",
			cmdfunc = function ()
				local r = BCF_GetOption("SHOWDOTHEAL");
				if ( r == 1 ) then
					BCF_SetOption("SHOWDOTHEAL", 0);
				else
					BCF_SetOption("SHOWDOTHEAL", 1);
				end
			end,
			default = 1,
		},
		-- 이 메세지의 옵션은 "자신의 스킬에 의한 자신의 체력 회복 메세지(지속적)"에서 이미 설정했으므로 command를 또 설정할 필요없고 commandkey만 동일하게 지정하면 됨
	});

	-- 파티원의 스킬에 의한 파워회복
	BCF_Register({
		event = "CHAT_MSG_SPELL_PERIODIC_PARTY_BUFFS",
		pattern = "당신은 (.+)의 (.+)|1으로;로; 인해 (%d+)의 (.+)|1을;를; 얻었습니다.",
		showmsg = 0,
		callback = function(event,msg,target,spell,power,ptype)
			local rgb;
			local text;
			if ( BCF_GetOption("SHOWTYPECOLOR") == 1 ) then
				if ( ptype == "분노" ) then
					rgb = BCF_GetColor("GAINRAGE");
				elseif ( ptype == "마나" ) then
					rgb = BCF_GetColor("GAINMANA");
				elseif ( ptype == "기력" ) then
					rgb = BCF_GetColor("GAINENERGY");
				else
					rgb = BCF_GetColor("GAINPOWER");
				end
			else
				rgb = BCF_GetColor("GAINPOWER");
			end
			text = "+"..power.." "..ptype;
			BCF_DisplayMessage({
				text = text,
				rgb = rgb,
				fontsize = BCF_GetOption("FONTSIZE"),
				paneltype = "fountain",
			});
		end,
		commandkey = "SHOWGAINPOWER",
	});


-----------------------------------------
-- CHAT_MSG_SPELL_PERIODIC_CREATURE_BUFFS
-----------------------------------------

	-- 자신의 스킬에 의한 npc의 체력 회복 메시지
	BCF_Register({
		event = "CHAT_MSG_SPELL_PERIODIC_CREATURE_BUFFS",
		pattern = "(.+)|1이;가; 당신의 (.+)|1으로;로; 인해 생명력이 (%d+)만큼 회복되었습니다.",
		showmsg = 0,
		callback = function(event,msg,target,spell,heal)
			local healing = heal;
			if ( BCF_GetOption("SHOWOVERHEAL") == 1 ) then
				healing = BCF_GetOverheal(target,heal);
			end
			local text;
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				text = target..": +"..healing.."("..spell..")";
			else
				text = target..": +"..healing;
			end
			BCF_DisplayMessage({
				text = text,
				fontheight = BCF_GetOption("FONTSIZE"),
				rgb = BCF_GetColor("HEAL"),
				paneltype = "fountain",
				font = "Fonts\\K_Damage.TTF",
			});
		end,
		commandkey = "SHOWDOTHEAL",
		-- 이 메세지의 옵션은 "자신의 스킬에 의한 자신의 체력 회복 메세지(지속적)"에서 이미 설정했으므로 command를 또 설정할 필요없고 commandkey만 동일하게 지정하면 됨
	});

	-- 파워회복(틱) 자신의 스킬 (토템)
	BCF_Register({
		event = "CHAT_MSG_SPELL_PERIODIC_CREATURE_BUFFS",
		pattern = "당신은 (.+)|1으로;로; 인해 (%d+)의 (.+)|1을;를; 얻었습니다.",
		showmsg = 0,
		callback = function(event,msg,spell,power,ptype)
			local rgb;
			local text;
			if ( BCF_GetOption("SHOWTYPECOLOR") == 1 ) then
				if ( ptype == "분노" ) then
					rgb = BCF_GetColor("GAINRAGE");
				elseif ( ptype == "마나" ) then
					rgb = BCF_GetColor("GAINMANA");
				elseif ( ptype == "기력" ) then
					rgb = BCF_GetColor("GAINENERGY");
				else
					rgb = BCF_GetColor("GAINPOWER");
				end
			else
				rgb = BCF_GetColor("GAINPOWER");
			end
			text = "+"..power.." "..ptype;
			BCF_DisplayMessage({
				text = text,
				rgb = rgb,
				fontsize = BCF_GetOption("FONTSIZE"),
				paneltype = "fountain",
			});
		end,
		commandkey = "SHOWGAINPOWER",
	});

	-- 파워회복 소환수의 스킬
	BCF_Register({
		event = "CHAT_MSG_SPELL_PET_BUFF",
		pattern = "당신은 (.-)의 (.+)|1으로;로; 인해 (%d+)의 (.+)|1을;를; 얻었습니다.",
		showmsg = 0,
		callback = function(event,msg,target,spell,power,ptype)
			local rgb;
			local text;
			if ( BCF_GetOption("SHOWTYPECOLOR") == 1 ) then
				if ( ptype == "분노" ) then
					rgb = BCF_GetColor("GAINRAGE");
				elseif ( ptype == "마나" ) then
					rgb = BCF_GetColor("GAINMANA");
				elseif ( ptype == "기력" ) then
					rgb = BCF_GetColor("GAINENERGY");
				else
					rgb = BCF_GetColor("GAINPOWER");
				end
			else
				rgb = BCF_GetColor("GAINPOWER");
			end
			text = "+"..power.." "..ptype;
			BCF_DisplayMessage({
				text = text,
				rgb = rgb,
				fontsize = BCF_GetOption("FONTSIZE"),
				paneltype = "fountain",
			});
		end,
		commandkey = "SHOWGAINPOWER",
	});


-----------------------------------------------
-- CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_BUFFS
-----------------------------------------------

	-- 자신의 스킬에 의한 타인의 체력 회복 메시지(지속적)	
	BCF_Register({
		event = "CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_BUFFS",
		pattern = "(.+)|1이;가; 당신의 (.+)|1으로;로; 인해 생명력이 (%d+)만큼 회복되었습니다.",
		showmsg = 0,
		callback = function(event,msg,target,spell,heal)
			local healing = heal;
			if ( BCF_GetOption("SHOWOVERHEAL") == 1 ) then
				healing = BCF_GetOverheal(target,heal);
			end
			local text;
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				text = target..": +"..healing.."("..spell..")";
			else
				text = target..": +"..healing;
			end
			BCF_DisplayMessage({
				text = text,
				fontheight = BCF_GetOption("FONTSIZE"),
				rgb = BCF_GetColor("HEAL"),
				paneltype = "fountain",
				font = "Fonts\\K_Damage.TTF",
			});
		end,
		commandkey = "SHOWDOTHEAL",
		-- 이 메세지의 옵션은 "자신의 스킬에 의한 자신의 체력 회복 메세지(지속적)"에서 이미 설정했으므로 command를 또 설정할 필요없고 commandkey만 동일하게 지정하면 됨
	});


	-- 타인의 스킬에 의한 파워회복
	BCF_Register({
		event = "CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_BUFFS",
		pattern = "당신은 (.-)의 (.+)|1으로;로; 인해 (%d+)의 (.+)|1을;를; 얻었습니다.",
		showmsg = 0,
		callback = function(event,msg,target,spell,power,ptype)
			local rgb;
			local text;
			if ( BCF_GetOption("SHOWTYPECOLOR") == 1 ) then
				if ( ptype == "분노" ) then
					rgb = BCF_GetColor("GAINRAGE");
				elseif ( ptype == "마나" ) then
					rgb = BCF_GetColor("GAINMANA");
				elseif ( ptype == "기력" ) then
					rgb = BCF_GetColor("GAINENERGY");
				else
					rgb = BCF_GetColor("GAINPOWER");
				end
			else
				rgb = BCF_GetColor("GAINPOWER");
			end
			text = "+"..power.." "..ptype;
			BCF_DisplayMessage({
				text = text,
				rgb = rgb,
				fontsize = BCF_GetOption("FONTSIZE"),
				paneltype = "fountain",
			});
		end,
		commandkey = "SHOWGAINPOWER",
	});


---------------------------
-- CHAT_MSG_SPELL_SELF_BUFF
---------------------------

	-- 자신의 스킬에 의한 체력 회복 메세지 - 0205
	BCF_Register({
		event = "CHAT_MSG_SPELL_SELF_BUFF",
		pattern = "당신은 자신의 (.+)|1으로;로; 생명력이 (%d+)만큼 회복되었습니다.",
		showmsg = 0,
		callback = function(event,msg,spell,heal)
			local healing = heal;
			if ( BCF_GetOption("SHOWOVERHEAL") == 1 ) then
				healing = BCF_GetOverheal("player",heal);
			end
			local text;
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				text = spell..": +"..healing;
			else
				text = "+"..healing;
			end
			BCF_DisplayMessage({
				text = text,
				fontheight = BCF_GetOption("FONTSIZE"),
				rgb = BCF_GetColor("HEAL"),
				paneltype = "fountain",
				font = "Fonts\\K_Damage.TTF",
			});
		end,
		commandkey = "SHOWHEAL",
		-- 이 메세지의 옵션은 "자신의 스킬에 의한 자신의 체력 회복 메세지(지속적)"에서 이미 설정했으므로 command를 또 설정할 필요없고 commandkey만 동일하게 지정하면 됨
	});

	-- 자신의 스킬에 의한 체력 회복 극대화 메세지 - 0208
	BCF_Register({
		event = "CHAT_MSG_SPELL_SELF_BUFF",
		pattern = "당신은 자신의 (.+)|1이;가; 극대화 효과를 발휘하여 생명력이 (%d+)만큼 회복되었습니다.",
		showmsg = 0,
		callback = function(event,msg,spell,heal)
			local healing = heal;
			if ( BCF_GetOption("SHOWOVERHEAL") == 1 ) then
				healing = BCF_GetOverheal("player",heal);
			end
			local text;
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				text = spell..": +"..healing;
			else
				text = "+"..healing;
			end
			BCF_DisplayMessage({
				text = text,
				fontheight = BCF_GetOption("FONTSIZE") * 1.25,
				rgb = BCF_GetColor("HEAL"),
				paneltype = "crit",
				font = "Fonts\\K_Damage.TTF",
			});
		end,
		commandkey = "SHOWHEAL",
		-- 이 메세지의 옵션은 "자신의 스킬에 의한 자신의 체력 회복 메세지(지속적)"에서 이미 설정했으므로 command를 또 설정할 필요없고 commandkey만 동일하게 지정하면 됨
	});

	-- 스킬로 다른이를 치유하는 메세지 : [스킬]로 [대상]의 생명력이 [얼마]만큼 회복되었습니다.
	BCF_Register({
		event = "CHAT_MSG_SPELL_SELF_BUFF",
		pattern = "당신의 (.+)|1으로;로; (.+)의 생명력이 (%d+)만큼 회복되었습니다.",
		callback = function(event,msg,spell,target,heal)
			local healing = heal;
			if ( BCF_GetOption("SHOWOVERHEAL") == 1 ) then
				healing = BCF_GetOverheal(target,heal);
			end
			local text;
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				text = target..": +"..healing.."("..spell..")";
			else
				text = target..": +"..healing;
			end
			BCF_DisplayMessage({
				text = text,
				fontheight = BCF_GetOption("FONTSIZE"),
				rgb = BCF_GetColor("HEAL"),
				paneltype = "fountain",
				font = "Fonts\\K_Damage.TTF",
			});
		end,
		commandkey = "HEALAMOUNT",
		command = {
			title = "힐량 메세지",
			cmd = {"힐량","healamount"},
			helpmsg = "%s: 자신의 힐량을 보여줍니다.",
			statusmsg = " 힐량 표시 메세지 : %s",
			cmdfunc = function ()
				local r = BCF_GetOption("HEALAMOUNT");
				if ( r == 1 ) then
					BCF_SetOption("HEALAMOUNT", 0);
				else
					BCF_SetOption("HEALAMOUNT", 1);
				end
			end,
			default = 1,
		},
	});

	-- 스킬로 다른이를 치유하는 극대화 메세지 : [스킬]로 [대상]의 생명력이 [얼마]만큼 회복되었습니다.
	BCF_Register({
		event = "CHAT_MSG_SPELL_SELF_BUFF",
		pattern = "당신의 (.+)|1이;가; 극대화 효과를 발휘하여 (.+)의 생명력이 (%d+)만큼 회복되었습니다.",
		callback = function(event,msg,spell,target,heal)
			local healing = heal;
			if ( BCF_GetOption("SHOWOVERHEAL") == 1 ) then
				healing = BCF_GetOverheal(target,heal);
			end
			local text;
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				text = target..": +"..healing.."("..spell..")";
			else
				text = target..": +"..healing;
			end
			BCF_DisplayMessage({
				text = text,
				fontheight = BCF_GetOption("FONTSIZE"),
				rgb = BCF_GetColor("HEAL"),
				paneltype = "crit",
				font = "Fonts\\K_Damage.TTF",
			});
		end,
		commandkey = "HEALAMOUNT",
	});

	-- 파워회복
	BCF_Register({
		event = "CHAT_MSG_SPELL_SELF_BUFF",
		pattern = "당신은 (.+)|1으로;로; 인해 (%d+)의 (.+)|1을;를; 얻었습니다.",
		showmsg = 0,
		callback = function(event,msg,spell,power,ptype)
			local rgb;
			local text;
			if ( BCF_GetOption("SHOWTYPECOLOR") == 1 ) then
				if ( ptype == "분노" ) then
					rgb = BCF_GetColor("GAINRAGE");
				elseif ( ptype == "마나" ) then
					rgb = BCF_GetColor("GAINMANA");
				elseif ( ptype == "기력" ) then
					rgb = BCF_GetColor("GAINENERGY");
				else
					rgb = BCF_GetColor("GAINPOWER");
				end
			else
				rgb = BCF_GetColor("GAINPOWER");
			end
			text = "+"..power.." "..ptype;
			BCF_DisplayMessage({
				text = text,
				rgb = rgb,
				fontsize = BCF_GetOption("FONTSIZE"),
				paneltype = "fountain",
			});
		end,
		commandkey = "SHOWGAINPOWER",
	});


------------------------------------
-- CHAT_MSG_SPELL_HOSTILEPLAYER_BUFF
------------------------------------

	-- 타인의 스킬에 의한 체력 회복 메세지 - 0205
	BCF_Register({
		event = "CHAT_MSG_SPELL_HOSTILEPLAYER_BUFF",
--		pattern = "(.[^%s]+)의 (.+)|1으로;로; 당신의 생명력이 (%d+)만큼 회복되었습니다.",
		pattern = "(.+)|1으로;로; 당신의 생명력이 (%d+)만큼 회복되었습니다.",
		showmsg = 0,
		callback = function(event,msg,spell,heal)
			local healing = heal;
			if ( BCF_GetOption("SHOWOVERHEAL") == 1 ) then
				healing = BCF_GetOverheal("player",heal);
			end
			local text;
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				local who,spell2
				if SW_FixLogStrings then
					_,_,who,spell2 = string.find(spell,"(.+) 의 (.+)");
				else
					_,_,who,spell2 = string.find(spell,"(.-)의 (.+)");
				end
				if ( who and spell2 ) then
					text = who..": +"..healing.."("..spell2..")";
				else
					text = spell..": +"..healing;
				end
			else
				text = "+"..healing;
			end
			BCF_DisplayMessage({
				text = text,
				fontheight = BCF_GetOption("FONTSIZE"),
				rgb = BCF_GetColor("HEAL"),
				paneltype = "fountain",
				font = "Fonts\\K_Damage.TTF",
			});
		end,
		commandkey = "SHOWHEAL",
		-- 이 메세지의 옵션은 "자신의 스킬에 의한 체력 회복 메세지(지속적)"에서 이미 설정했으므로 command를 또 설정할 필요없고 commandkey만 동일하게 지정하면 됨
	});

	-- 타인의 스킬에 의한 체력 회복 극대화 메세지 - 0205
	BCF_Register({
		event = "CHAT_MSG_SPELL_HOSTILEPLAYER_BUFF",
		pattern = "(.+)|1이;가; 극대화 효과를 발휘하여 당신의 생명력이 (%d+)만큼 회복되었습니다.",
		showmsg = 0,
		callback = function(event,msg,spell,heal)
			local healing = heal;
			if ( BCF_GetOption("SHOWOVERHEAL") == 1 ) then
				healing = BCF_GetOverheal("player",heal);
			end
			local text;
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				local who,spell2
				if SW_FixLogStrings then
					_,_,who,spell2 = string.find(spell,"(.+) 의 (.+)");
				else
					_,_,who,spell2 = string.find(spell,"(.-)의 (.+)");
				end

				if ( who and spell2 ) then
					text = who..": +"..healing.."("..spell2..")";
				else
					text = spell..": +"..healing;
				end
			else
				text = "+"..healing;
			end
			BCF_DisplayMessage({
				text = text,
				fontheight = BCF_GetOption("FONTSIZE"),
				rgb = BCF_GetColor("HEAL"),
				paneltype = "crit",
				font = "Fonts\\K_Damage.TTF",
			});
		end,
		commandkey = "SHOWHEAL",
		-- 이 메세지의 옵션은 "자신의 스킬에 의한 체력 회복 메세지(지속적)"에서 이미 설정했으므로 command를 또 설정할 필요없고 commandkey만 동일하게 지정하면 됨
	});


--------------------------------------
-- CHAT_MSG_SPELL_PERIODIC_SELF_DAMAGE
--------------------------------------

	-- 디버프 메세지(속성표시) - 독,저주,질병,마법 - 0205
	BCF_Register({
		event = "CHAT_MSG_SPELL_PERIODIC_SELF_DAMAGE",
		pattern = "당신은 (.+)에 걸렸습니다.$",
		showmsg = 0,
		callback = function(event,msg,debuffname)
			local i = 1;
			local rgb;
			local debuff, debuffCount, debufftype = UnitDebuff("player",i);
			local localDT = "";
			while ( debuff ) do
				GameTooltip:ClearLines();
				GameTooltip:SetUnitDebuff("player",i);
				local name = getglobal("GameTooltipTextLeft1");
				if ( name and name:IsShown() and name:GetText() ~= nil ) then
					nametext = name:GetText();
					if ( nametext and nametext == debuffname ) then
						break;
					end
				end
				i = i + 1;
				debuff, debuffCount, debufftype = UnitDebuff("player",i);
			end
			if ( debufftype and DebuffTypeColor[debufftype] ) then
				localDT = ":"..BCFG_DEBUFF_LOCALIZE[debufftype];
				rgb = DebuffTypeColor[debufftype];
			else
				rgb = DebuffTypeColor["none"];
			end
			BCF_DisplayMessage({
				text = "+["..debuffname..localDT.."]",
				fontheight = BCF_GetOption("FONTSIZE"),
				rgb = rgb,
				paneltype = "standard",
				font = "Fonts\\2002b.TTF",
				outline = "OUTLINE"
			});
		end,
		commandkey = "SHOWDEBUFFING",
		command = {
			title = "디버프 메세지",
			cmd = {"디버프","debuffing"},
			helpmsg = "%s: 디버프 효과를 얻을때 메세지를 보여줍니다.",
			statusmsg = " 디버프 메세지 : %s",
			cmdfunc = function ()
				local r = BCF_GetOption("SHOWDEBUFFING");
				if ( r == 1 ) then
					BCF_SetOption("SHOWDEBUFFING", 0);
				else
					BCF_SetOption("SHOWDEBUFFING", 1);
				end
			end,
		},
	});

	-- 디버프 메세지(속성표시) - 독,저주,질병,마법 x n개 - 0205
	BCF_Register({
		event = "CHAT_MSG_SPELL_PERIODIC_SELF_DAMAGE",
		pattern = "당신은 (.+)에 걸렸습니다. %((%d+)%)",
		showmsg = 0,
		callback = function(event,msg,debuffname,appcount)
			local i = 1;
			local rgb;
			local debuff, debuffCount, debufftype = UnitDebuff("player",i);
			local localDT = "";
			while ( debuff ) do
				GameTooltip:ClearLines();
				GameTooltip:SetUnitDebuff("player",i);
				local name = getglobal("GameTooltipTextLeft1");
				if ( name and name:IsShown() and name:GetText() ~= nil ) then
					nametext = name:GetText();
					if ( nametext and nametext == debuffname ) then
						break;
					end
				end
				i = i + 1;
				debuff, debuffCount, debufftype = UnitDebuff("player",i);
			end
			if ( debufftype and DebuffTypeColor[debufftype] ) then
				localDT = ":"..BCFG_DEBUFF_LOCALIZE[debufftype];
				rgb = DebuffTypeColor[debufftype];
			else
				rgb = DebuffTypeColor["none"];
			end
			BCF_DisplayMessage({
				text = "+["..debuffname..localDT.."] x "..appcount,
				fontheight = BCF_GetOption("FONTSIZE"),
				rgb = rgb,
				paneltype = "standard",
				font = "Fonts\\2002b.TTF",
				outline = "OUTLINE"
			});
		end,
		commandkey = "SHOWDEBUFFING",
	});

	-- dot self데미지 - 0205
	BCF_Register({
		event = "CHAT_MSG_SPELL_PERIODIC_SELF_DAMAGE",
		pattern = "당신은 (.+)에 의해 (%d+)의 ([신성|화염|자연|냉기|암흑|비전|물리]+) 피해를 입었습니다.",
		commandkey = "SHOWDAMAGE",
		callback = function (event,msg,spell,damage,atype)
			local text;
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				local who,spell2;
				if SW_FixLogStrings then
					_,_,who,spell2 = string.find(spell,"(.+) 의 (.+)");
				else
					_,_,who,spell2 = string.find(spell,"(.-)의 (.+)");
				end
				if ( who and spell2 ) then
					if ( BCF_GetOption("SHOWSPELLSCHOOL") == 1 ) then
						text = "-"..damage.."["..spell2..":"..atype.."]";
					else
						text = "-"..damage.."["..spell2.."]";
					end
				else
					if ( BCF_GetOption("SHOWSPELLSCHOOL") == 1 ) then
						text = "-"..damage.."["..spell..":"..atype.."]";
					else
						text = "-"..damage.."["..spell.."]";
					end
				end
			else
				text = "-"..damage;
			end

			local suffix = BCF_FindSuffix(msg);

			local rgb;
			if ( BCF_GetOption("SHOWTYPECOLOR") == 1 ) then
				if ( atype == "신성" ) then
					rgb = BCF_GetColor("Holy");
				elseif ( atype == "화염" ) then
					rgb = BCF_GetColor("Fire");
				elseif ( atype == "자연" ) then
					rgb = BCF_GetColor("Nature");
				elseif ( atype == "냉기" ) then
					rgb = BCF_GetColor("Frost");
				elseif ( atype == "암흑" ) then
					rgb = BCF_GetColor("Shadow");
				elseif ( atype == "비전" ) then
					rgb = BCF_GetColor("Arcane");
				else
					rgb = BCF_GetColor("SPELLDAMAGE");
				end
			else
				rgb = BCF_GetColor("SPELLDAMAGE");
			end
			BCF_DisplayMessage({
				text = text..suffix,
				rgb = rgb,
				paneltype = "standard",
				isStaggered = 1,
			});
		end,
	});


-----------------------------
-- CHAT_MSG_COMBAT_HONOR_GAIN
-----------------------------

---------------------------------
-- CHAT_MSG_COMBAT_FACTION_CHANGE
---------------------------------

-----------------------------
-- CHAT_MSG_SPELL_SELF_DAMAGE
-----------------------------

	BCF_Register({ -- 0205
		event = "CHAT_MSG_SPELL_SELF_DAMAGE",
		pattern = "공격했지만 교묘히 피했습니다.$",
		msg = "<제압발사>",
		rgb = BCF_GetColor("SPELLACTIVE"),
		fontheight = BCF_GetOption("FONTSIZE"),
		soundfile = "Interface\\AddOns\\BlinkCombatFeedback\\sound\\Roar.wav",
		showmsg = 1,
		paneltype = "sticky",
		font = "Fonts\\2002b.TTF",
		commandkey = "SHOWSPELLACTIVE",
		class = "전사",
	});


------------------------------
-- CHAT_MSG_COMBAT_SELF_MISSES
------------------------------

	BCF_Register({ -- 0205
		event = "CHAT_MSG_COMBAT_SELF_MISSES",
		pattern = "공격했지만 교묘히 피했습니다.$",
		msg = "<제압발사>",
		rgb = BCF_GetColor("SPELLACTIVE"),
		fontheight = BCF_GetOption("FONTSIZE"),
		soundfile = "Interface\\AddOns\\BlinkCombatFeedback\\sound\\Roar.wav",
		showmsg = 1,
		paneltype = "sticky",
		font = "Fonts\\2002b.TTF",
		commandkey = "SHOWSPELLACTIVE",
		class = "전사",
	});


----------------------------
-- CHAT_MSG_COMBAT_SELF_HITS
----------------------------

	-- 숨막힘데미지
	BCF_Register({
		event = "CHAT_MSG_COMBAT_SELF_HITS",
		pattern = "당신은 숨을 쉴 수 없어 (%d+)의 피해를 입었습니다.",
		msg = "-%1 (숨막힘)",
		rgb = BCF_GetColor("NORMALDAMAGE"),
		paneltype = "standard",
		isStaggered = 1,
		commandkey = "SHOWDAMAGE",
		command = {
			title = "피해량 표시",
			cmd = {"피해량","damage"},
			helpmsg = "%s: 피해량을 표시해줍니다.",
			statusmsg = " 피해량 표시 : %s",
			cmdfunc = function ()
				local r = BCF_GetOption("SHOWDAMAGE");
				if ( r == 1 ) then
					BCF_SetOption("SHOWDAMAGE", 0);
				else
					BCF_SetOption("SHOWDAMAGE", 1);
				end
			end,
		},
	});

	-- 낙하데미지 - 0205
	BCF_Register({
		event = "CHAT_MSG_COMBAT_SELF_HITS",
		pattern = "당신은 낙하할 때의 충격으로 (%d+)의 피해를 입었습니다.",
		msg = "-%1 (낙하)",
		rgb = BCF_GetColor("NORMALDAMAGE"),
		paneltype = "standard",
		isStaggered = 1,
		commandkey = "SHOWDAMAGE",
	});

	-- 화염데미지 - 0205
	BCF_Register({
		event = "CHAT_MSG_COMBAT_SELF_HITS",
		pattern = "당신은 (%d+)의 화염 피해를 입었습니다.",
		msg = "-%1 (화염)",
		rgb = BCF_GetColor("NORMALDAMAGE"),
		paneltype = "standard",
		isStaggered = 1,
		commandkey = "SHOWDAMAGE",
	});

	-- 용암데미지 - 0208
	BCF_Register({
		event = "CHAT_MSG_COMBAT_SELF_HITS",
		pattern = "당신은 용암의 열기로 인해 (%d+)의 피해를 입었습니다.",
		msg = "-%1 (용암)",
		rgb = BCF_GetColor("NORMALDAMAGE"),
		paneltype = "standard",
		isStaggered = 1,
		commandkey = "SHOWDAMAGE",
	});

	-- 피로데미지 - 0208
	BCF_Register({
		event = "CHAT_MSG_COMBAT_SELF_HITS",
		pattern = "당신은 너무 기진맥진하여 (%d+)의 피해를 입었습니다.",
		msg = "-%1 (피로)",
		rgb = BCF_GetColor("NORMALDAMAGE"),
		paneltype = "standard",
		isStaggered = 1,
		commandkey = "SHOWDAMAGE",
	});

	-- 독성데미지 - 0208
	BCF_Register({
		event = "CHAT_MSG_COMBAT_SELF_HITS",
		pattern = "당신은 독성으로 인해 (%d+)의 피해를 입었습니다.",
		msg = "-%1 (독성)",
		rgb = BCF_GetColor("NORMALDAMAGE"),
		paneltype = "standard",
		isStaggered = 1,
		commandkey = "SHOWDAMAGE",
	});

	-- 데미지 - 있는거맞나? - 0208
	BCF_Register({
		event = "CHAT_MSG_COMBAT_SELF_HITS",
		pattern = "당신은 (.+)에 의해 (%d+)의 (.+) 피해를 입었습니다.",
		callback = function (event,msg,spell,damage,damagetype)
			local text;
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				if ( BCF_GetOption("SHOWSPELLSCHOOL") == 1 ) then
					text = "-"..damage.." ("..spell..":"..damagetype..")";
				else
					text = "-"..damage.." ("..spell..")";
				end
			else
				text = "-"..damage;
			end
			local suffix = BCF_FindSuffix(msg);
			BCF_DisplayMessage({
				text = text..suffix,
				rgb = BCF_GetColor("NORMALDAMAGE"),
				paneltype = "standard",
				isStaggered = 1,
			});
		end,
		commandkey = "SHOWDAMAGE",
	});


----------------------------------------
-- CHAT_MSG_COMBAT_CREATURE_VS_SELF_HITS
----------------------------------------

	-- 속성데미지
	BCF_Register({
		event = "CHAT_MSG_COMBAT_CREATURE_VS_SELF_HITS",
		pattern = "(.+)|1이;가; 당신을 공격하여 (%d+)의 ([신성|화염|자연|냉기|암흑|비전|물리]+) 피해를 입혔습니다.",
		callback = function (event,msg,target,damage,atype)
			local text;
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				if ( BCF_GetOption("SHOWSPELLSCHOOL") == 1 ) then
					text = "-"..damage.."["..atype.."]";
				else
					text = "-"..damage;
				end
			else
				text = "-"..damage;
			end
			local suffix = BCF_FindSuffix(msg);
			local rgb;
			if ( BCF_GetOption("SHOWTYPECOLOR") == 1 ) then
				if ( atype == "신성" ) then
					rgb = BCF_GetColor("Holy");
				elseif ( atype == "화염" ) then
					rgb = BCF_GetColor("Fire");
				elseif ( atype == "자연" ) then
					rgb = BCF_GetColor("Nature");
				elseif ( atype == "냉기" ) then
					rgb = BCF_GetColor("Frost");
				elseif ( atype == "암흑" ) then
					rgb = BCF_GetColor("Shadow");
				elseif ( atype == "비전" ) then
					rgb = BCF_GetColor("Arcane");
				else
					rgb = BCF_GetColor("NORMALDAMAGE");
				end
			else
				rgb = BCF_GetColor("NORMALDAMAGE");
			end
			BCF_DisplayMessage({
				text = text..suffix,
				rgb = rgb,
				paneltype = "standard",
				isStaggered = 1,
			});
		end,
		commandkey = "SHOWDAMAGE",
	});

	-- 속성데미지 치명타
	BCF_Register({
		event = "CHAT_MSG_COMBAT_CREATURE_VS_SELF_HITS",
		pattern = "(.+)|1이;가; 당신을 공격하여 (%d+)의 ([신성|화염|자연|냉기|암흑|비전|물리]+) 치명상 피해를 입혔습니다.",
		callback = function (event,msg,target,damage,atype)
			local text;
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				if ( BCF_GetOption("SHOWSPELLSCHOOL") == 1 ) then
					text = "-"..damage.."["..atype.."]";
				else
					text = "-"..damage;
				end
			else
				text = "-"..damage;
			end
			local suffix = BCF_FindSuffix(msg);
			local rgb;
			if ( BCF_GetOption("SHOWTYPECOLOR") == 1 ) then
				if ( atype == "신성" ) then
					rgb = BCF_GetColor("Holy");
				elseif ( atype == "화염" ) then
					rgb = BCF_GetColor("Fire");
				elseif ( atype == "자연" ) then
					rgb = BCF_GetColor("Nature");
				elseif ( atype == "냉기" ) then
					rgb = BCF_GetColor("Frost");
				elseif ( atype == "암흑" ) then
					rgb = BCF_GetColor("Shadow");
				elseif ( atype == "비전" ) then
					rgb = BCF_GetColor("Arcane");
				else
					rgb = BCF_GetColor("NORMALDAMAGE");
				end
			else
				rgb = BCF_GetColor("NORMALDAMAGE");
			end
			BCF_DisplayMessage({
				text = text..suffix,
				rgb = rgb,
				paneltype = "crit",
				isStaggered = 1,
			});
		end,
		commandkey = "SHOWDAMAGE",
	});

	-- 데미지 - 0205
	BCF_Register({
		event = "CHAT_MSG_COMBAT_CREATURE_VS_SELF_HITS",
		pattern = "(.+)|1이;가; 당신을 공격하여 (%d+)의 피해를 입혔습니다.",
		callback = function (event,msg,target,damage)
			local text = "-"..damage..BCF_FindSuffix(msg);
			BCF_DisplayMessage({
				text = text,
				rgb = BCF_GetColor("NORMALDAMAGE"),
				paneltype = "standard",
				isStaggered = 1,
			});
		end,
		commandkey = "SHOWDAMAGE",
	});

	-- 데미지 치명타 - 0205
	BCF_Register({
		event = "CHAT_MSG_COMBAT_CREATURE_VS_SELF_HITS",
		pattern = "(.+)|1이;가; 당신을 공격하여 (%d+)의 치명상 피해를 입혔습니다.",
		callback = function (event,msg,target,damage)
			local text = "-"..damage..BCF_FindSuffix(msg);
			BCF_DisplayMessage({
				text = text,
				rgb = BCF_GetColor("NORMALDAMAGE"),
				paneltype = "crit",
				isStaggered = 1,
			});
		end,
		commandkey = "SHOWDAMAGE",
	});


-----------------------------------------
-- CHAT_MSG_SPELL_CREATURE_VS_SELF_DAMAGE
-----------------------------------------

	-- 속성데미지 - 0205
	BCF_Register({
		event = "CHAT_MSG_SPELL_CREATURE_VS_SELF_DAMAGE",
		pattern = "(.+)|1이;가; (.+)|1으로;로; 당신에게 (%d+)의 ([신성|화염|자연|냉기|암흑|비전|물리]+) 피해를 입혔습니다.",
		callback = function (event,msg,target,spell,damage,atype)
			local text;
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				if ( BCF_GetOption("SHOWSPELLSCHOOL") == 1 ) then
					text = "-"..damage.."["..spell..":"..atype.."]";
				else
					text = "-"..damage.."["..spell.."]";
				end
			else
				text = "-"..damage;
			end
			local suffix = BCF_FindSuffix(msg);
			local rgb;
			if ( BCF_GetOption("SHOWTYPECOLOR") == 1 ) then
				if ( atype == "신성" ) then
					rgb = BCF_GetColor("Holy");
				elseif ( atype == "화염" ) then
					rgb = BCF_GetColor("Fire");
				elseif ( atype == "자연" ) then
					rgb = BCF_GetColor("Nature");
				elseif ( atype == "냉기" ) then
					rgb = BCF_GetColor("Frost");
				elseif ( atype == "암흑" ) then
					rgb = BCF_GetColor("Shadow");
				elseif ( atype == "비전" ) then
					rgb = BCF_GetColor("Arcane");
				else
					rgb = BCF_GetColor("SPELLDAMAGE");
				end
			else
				rgb = BCF_GetColor("SPELLDAMAGE");
			end
			BCF_DisplayMessage({
				text = text..suffix,
				rgb = rgb,
				paneltype = "standard",
				isStaggered = 1,
			});
		end,
		commandkey = "SHOWDAMAGE",
	});

	-- 속성데미지 치명타 - 0205
	BCF_Register({
		event = "CHAT_MSG_SPELL_CREATURE_VS_SELF_DAMAGE",
		pattern = "(.+)|1이;가; (.+)|1으로;로; 당신에게 (%d+)의 ([신성|화염|자연|냉기|암흑|비전|물리]+) 치명상 피해를 입혔습니다.",
		callback = function (event,msg,target,spell,damage,atype)
			local text;
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				if ( BCF_GetOption("SHOWSPELLSCHOOL") == 1 ) then
					text = "-"..damage.."["..spell..":"..atype.."]";
				else
					text = "-"..damage.."["..spell.."]";
				end
			else
				text = "-"..damage;
			end
			local suffix = BCF_FindSuffix(msg);
			local rgb;
			if ( BCF_GetOption("SHOWTYPECOLOR") == 1 ) then
				if ( atype == "신성" ) then
					rgb = BCF_GetColor("Holy");
				elseif ( atype == "화염" ) then
					rgb = BCF_GetColor("Fire");
				elseif ( atype == "자연" ) then
					rgb = BCF_GetColor("Nature");
				elseif ( atype == "냉기" ) then
					rgb = BCF_GetColor("Frost");
				elseif ( atype == "암흑" ) then
					rgb = BCF_GetColor("Shadow");
				elseif ( atype == "비전" ) then
					rgb = BCF_GetColor("Arcane");
				else
					rgb = BCF_GetColor("SPELLDAMAGE");
				end
			else
				rgb = BCF_GetColor("SPELLDAMAGE");
			end
			BCF_DisplayMessage({
				text = text..suffix,
				rgb = rgb,
				paneltype = "crit",
				isStaggered = 1,
			});
		end,
		commandkey = "SHOWDAMAGE",
	});

	-- 데미지 - 0205
	BCF_Register({
		event = "CHAT_MSG_SPELL_CREATURE_VS_SELF_DAMAGE",
		pattern = "(.+)|1이;가; (.+)|1으로;로; 당신에게 (%d+)의 피해를 입혔습니다.",
		callback = function (event,msg,target,spell,damage)
			local text;
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				text = "-"..damage.."["..spell.."]"..BCF_FindSuffix(msg);
			else
				text = "-"..damage..BCF_FindSuffix(msg);
			end
			BCF_DisplayMessage({
				text = text,
				rgb = BCF_GetColor("SPELLDAMAGE"),
				paneltype = "standard",
				isStaggered = 1,
			});
		end,
		commandkey = "SHOWDAMAGE",
	});

	-- 데미지 - 0205
	BCF_Register({
		event = "CHAT_MSG_SPELL_CREATURE_VS_SELF_DAMAGE",
		pattern = "(.+)|1이;가; (.+)|1으로;로; 당신에게 (%d+)의 치명상 피해를 입혔습니다.",
		callback = function (event,msg,target,spell,damage)
			local text;
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				text = "-"..damage.."["..spell.."]"..BCF_FindSuffix(msg);
			else
				text = "-"..damage..BCF_FindSuffix(msg);
			end
			BCF_DisplayMessage({
				text = text,
				rgb = BCF_GetColor("SPELLDAMAGE"),
				paneltype = "crit",
				isStaggered = 1,
			});
		end,
		commandkey = "SHOWDAMAGE",
	});

-------------------------------------
-- CHAT_MSG_COMBAT_HOSTILEPLAYER_HITS
-------------------------------------

	-- 적플레이어의 평타 데미지 - 0205
	BCF_Register({
		event = "CHAT_MSG_COMBAT_HOSTILEPLAYER_HITS",
		pattern = "(.+)|1이;가; 당신을 공격하여 (%d+)의 피해를 입혔습니다.",
		callback = function (event,msg,target,damage)
			local text = "-"..damage..BCF_FindSuffix(msg);
			BCF_DisplayMessage({
				text = text,
				rgb = BCF_GetColor("NORMALDAMAGE"),
				paneltype = "standard",
				isStaggered = 1,
			});
		end,
		commandkey = "SHOWDAMAGE",
	});

	-- 적플레이어의 치명타 데미지 - 0205
	BCF_Register({
		event = "CHAT_MSG_COMBAT_HOSTILEPLAYER_HITS",
		pattern = "(.+)|1이;가; 당신을 공격하여 (%d+)의 치명상 피해를 입혔습니다.",
		callback = function (event,msg,target,damage)
			local text = "-"..damage..BCF_FindSuffix(msg);
			BCF_DisplayMessage({
				text = text,
				rgb = BCF_GetColor("NORMALDAMAGE"),
				paneltype = "crit",
			});
		end,
		commandkey = "SHOWDAMAGE",
	});

	-- 적플레이어의 자동사격 평타 데미지 - 0205
	BCF_Register({
		event = "CHAT_MSG_COMBAT_HOSTILEPLAYER_HITS",
		pattern = "(.+)|1이;가; (.+)|1으로;로; 당신에게 (%d+)의 피해를 입혔습니다.",
		callback = function (event,msg,target,spell,damage)
			local text = "-"..damage..BCF_FindSuffix(msg);
			BCF_DisplayMessage({
				text = text,
				rgb = BCF_GetColor("NORMALDAMAGE"),
				paneltype = "standard",
				isStaggered = 1,
			});
		end,
		commandkey = "SHOWDAMAGE",
	});

	-- 적플레이어의 자동사격 치명타 데미지 - 0205
	BCF_Register({
		event = "CHAT_MSG_COMBAT_HOSTILEPLAYER_HITS",
		pattern = "(.+)|1이;가; (.+)|1으로;로; 당신에게 (%d+)의 치명상 피해를 입혔습니다.",
		callback = function (event,msg,target,spell,damage)
			local text = "-"..damage..BCF_FindSuffix(msg);
			BCF_DisplayMessage({
				text = text,
				rgb = BCF_GetColor("NORMALDAMAGE"),
				paneltype = "crit",
			});
		end,
		commandkey = "SHOWDAMAGE",
	});

	-- 적플레이어의 마법봉 평타 데미지 - 0205
	BCF_Register({
		event = "CHAT_MSG_COMBAT_HOSTILEPLAYER_HITS",
		pattern = "(.+)|1이;가; (.+)|1으로;로; 당신에게 (%d+)의 ([신성|화염|자연|냉기|암흑|비전|물리]+) 피해를 입혔습니다.",
		callback = function (event,msg,target,spell,damage,damagetype)
			local text;
			if ( BCF_GetOption("SHOWSPELLSCHOOL") == 1 ) then
				text = "-"..damage.."["..damagetype.."]";
			else
				text = "-"..damage;
			end
			local suffix = BCF_FindSuffix(msg);
			local rgb;
			if ( BCF_GetOption("SHOWTYPECOLOR") == 1 ) then
				if ( damagetype == "신성" ) then
					rgb = BCF_GetColor("Holy");
				elseif ( damagetype == "화염" ) then
					rgb = BCF_GetColor("Fire");
				elseif ( damagetype == "자연" ) then
					rgb = BCF_GetColor("Nature");
				elseif ( damagetype == "냉기" ) then
					rgb = BCF_GetColor("Frost");
				elseif ( damagetype == "암흑" ) then
					rgb = BCF_GetColor("Shadow");
				elseif ( damagetype == "비전" ) then
					rgb = BCF_GetColor("Arcane");
				else
					rgb = BCF_GetColor("SPELLDAMAGE");
				end
			else
				rgb = BCF_GetColor("SPELLDAMAGE");
			end
			BCF_DisplayMessage({
				text = text..suffix,
				rgb = rgb,
				paneltype = "standard",
				isStaggered = 1,
			});
		end,
		commandkey = "SHOWDAMAGE",
	});

	-- 적플레이어의 마법봉 치명타 데미지 - 0205
	BCF_Register({
		event = "CHAT_MSG_COMBAT_HOSTILEPLAYER_HITS",
		pattern = "(.+)|1이;가; (.+)|1으로;로; 당신에게 (%d+)의 ([신성|화염|자연|냉기|암흑|비전|물리]+) 치명상 피해를 입혔습니다.",
		callback = function (event,msg,target,spell,damage,damagetype)
			local text;
			if ( BCF_GetOption("SHOWSPELLSCHOOL") == 1 ) then
				text = "-"..damage.."["..damagetype.."]";
			else
				text = "-"..damage;
			end
			local suffix = BCF_FindSuffix(msg);
			local rgb;
			if ( BCF_GetOption("SHOWTYPECOLOR") == 1 ) then
				if ( damagetype == "신성" ) then
					rgb = BCF_GetColor("Holy");
				elseif ( damagetype == "화염" ) then
					rgb = BCF_GetColor("Fire");
				elseif ( damagetype == "자연" ) then
					rgb = BCF_GetColor("Nature");
				elseif ( damagetype == "냉기" ) then
					rgb = BCF_GetColor("Frost");
				elseif ( damagetype == "암흑" ) then
					rgb = BCF_GetColor("Shadow");
				elseif ( damagetype == "비전" ) then
					rgb = BCF_GetColor("Arcane");
				else
					rgb = BCF_GetColor("SPELLDAMAGE");
				end
			else
				rgb = BCF_GetColor("SPELLDAMAGE");
			end
			BCF_DisplayMessage({
				text = text..suffix,
				rgb = rgb,
				paneltype = "crit",
			});
		end,
		commandkey = "SHOWDAMAGE",
	});

	--막타 메시지
	BCF_Register({
		event = "CHAT_MSG_COMBAT_HOSTILE_DEATH",
		pattern = "당신은 (.+)|1을;를; 처치했습니다!",
		callback = function (event,msg,target)
			local text;
			text = target.." 처치!";
			BCF_DisplayMessage({
				text = text,
				rgb = BCF_GetColor("NORMALDAMAGE"),
				paneltype = "crit",
				font = "Fonts\\2002b.TTF",
				outline = "OUTLINE",
			});
		end,
	});

--------------------------------------
-- CHAT_MSG_SPELL_HOSTILEPLAYER_DAMAGE
--------------------------------------

	-- 적플레이어의 속성 데미지 - 0205
	BCF_Register({
		event = "CHAT_MSG_SPELL_HOSTILEPLAYER_DAMAGE",
		pattern = "(.+)|1이;가; (.+)|1으로;로; 당신에게 (%d+)의 ([신성|화염|자연|냉기|암흑|비전|물리]+) 피해를 입혔습니다.",
		callback = function (event,msg,target,spell,damage,atype)
			local text;
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				if ( BCF_GetOption("SHOWSPELLSCHOOL") == 1 ) then
					text = "-"..damage.."["..spell..":"..atype.."]";
				else
					text = "-"..damage.."["..spell.."]";
				end
			else
				text = "-"..damage;
			end
			local suffix = BCF_FindSuffix(msg);
			local rgb;
			if ( BCF_GetOption("SHOWTYPECOLOR") == 1 ) then
				if ( atype == "신성" ) then
					rgb = BCF_GetColor("Holy");
				elseif ( atype == "화염" ) then
					rgb = BCF_GetColor("Fire");
				elseif ( atype == "자연" ) then
					rgb = BCF_GetColor("Nature");
				elseif ( atype == "냉기" ) then
					rgb = BCF_GetColor("Frost");
				elseif ( atype == "암흑" ) then
					rgb = BCF_GetColor("Shadow");
				elseif ( atype == "비전" ) then
					rgb = BCF_GetColor("Arcane");
				else
					rgb = BCF_GetColor("SPELLDAMAGE");
				end
			else
				rgb = BCF_GetColor("SPELLDAMAGE");
			end
			BCF_DisplayMessage({
				text = text..suffix,
				rgb = rgb,
				paneltype = "standard",
				isStaggered = 1,
			});
		end,
		commandkey = "SHOWDAMAGE",
	});

	-- 적플레이어의 속성 치명타 데미지 - 0205
	BCF_Register({
		event = "CHAT_MSG_SPELL_HOSTILEPLAYER_DAMAGE",
		pattern = "(.+)|1이;가; (.+)|1으로;로; 당신에게 (%d+)의 ([신성|화염|자연|냉기|암흑|비전|물리]+) 치명상 피해를 입혔습니다.",
		callback = function (event,msg,target,spell,damage,atype)
			local text;
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				if ( BCF_GetOption("SHOWSPELLSCHOOL") == 1 ) then
					text = "-"..damage.."["..spell..":"..atype.."]";
				else
					text = "-"..damage.."["..spell.."]";
				end
			else
				text = "-"..damage;
			end
			local suffix = BCF_FindSuffix(msg);
			local rgb;
			if ( BCF_GetOption("SHOWTYPECOLOR") == 1 ) then
				if ( atype == "신성" ) then
					rgb = BCF_GetColor("Holy");
				elseif ( atype == "화염" ) then
					rgb = BCF_GetColor("Fire");
				elseif ( atype == "자연" ) then
					rgb = BCF_GetColor("Nature");
				elseif ( atype == "냉기" ) then
					rgb = BCF_GetColor("Frost");
				elseif ( atype == "암흑" ) then
					rgb = BCF_GetColor("Shadow");
				elseif ( atype == "비전" ) then
					rgb = BCF_GetColor("Arcane");
				else
					rgb = BCF_GetColor("SPELLDAMAGE");
				end
			else
				rgb = BCF_GetColor("SPELLDAMAGE");
			end
			BCF_DisplayMessage({
				text = text..suffix,
				rgb = rgb,
				paneltype = "crit",
				isStaggered = 1,
			});
		end,
		commandkey = "SHOWDAMAGE",
	});

	-- 적플레이어의 스킬 데미지
	BCF_Register({
		event = "CHAT_MSG_SPELL_HOSTILEPLAYER_DAMAGE",
		pattern = "(.+)|1이;가; (.+)|1으로;로; 당신에게 (%d+)의 피해를 입혔습니다.",
		callback = function (event,msg,target,spell,damage)
			local text;
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				text = "-"..damage.."["..spell.."]"..BCF_FindSuffix(msg);
			else
				text = "-"..damage..BCF_FindSuffix(msg);
			end
			BCF_DisplayMessage({
				text = text,
				rgb = BCF_GetColor("SPELLDAMAGE"),
				paneltype = "standard",
				isStaggered = 1,
			});
		end,
		commandkey = "SHOWDAMAGE",
	});

	-- 적플레이어의 치명타 스킬 데미지
	BCF_Register({
		event = "CHAT_MSG_SPELL_HOSTILEPLAYER_DAMAGE",
		pattern = "(.+)|1이;가; (.+)|1으로;로; 당신에게 (%d+)의 치명상 피해를 입혔습니다.",
		callback = function (event,msg,target,spell,damage)
			local text;
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				text = "-"..damage.."["..spell.."]"..BCF_FindSuffix(msg);
			else
				text = "-"..damage..BCF_FindSuffix(msg);
			end
			BCF_DisplayMessage({
				text = text,
				rgb = BCF_GetColor("SPELLDAMAGE"),
				paneltype = "crit",
				isStaggered = 1,
			});
		end,
		commandkey = "SHOWDAMAGE",
	});


	-- 회복의 기원 : test
	BCF_Register({
		event = "CHAT_MSG_SPELL_HOSTILEPLAYER_BUFF",
		pattern = "회복의 기원|1으로;로; (.+)의 생명력이 (%d+)만큼 회복되었습니다.",
		showmsg = 0,
		callback = function(event,msg,target,heal)
			local pclass = UnitClass("player");
			local healing = heal;
			if ( BCF_GetOption("SHOWOVERHEAL") == 1 ) then
				healing = BCF_GetOverheal(target,heal);
			end
			local text;
			if (pclass == "사제") then
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				text = target..": +"..healing.."(회복의 기원)";
			else
				text = target..": +"..healing.."(회복의 기원)";
			end
			end
			BCF_DisplayMessage({
				text = text,
				fontheight = BCF_GetOption("FONTSIZE"),
				rgb = BCF_GetColor("HEAL"),
				paneltype = "fountain",				
				font = "Fonts\\K_Damage.TTF",
			});
		end,
		commandkey = "SHOWHEAL",		
	});

	-- 회복의 기원 : test
	BCF_Register({
		event = "CHAT_MSG_SPELL_HOSTILEPLAYER_BUFF",
		pattern = "회복의 기원|1이;가; 극대화 효과를 발휘하여 (.+)의 생명력이 (%d+)만큼 회복되었습니다.",
		showmsg = 0,
		callback = function(event,msg,target,heal)
			local pclass = UnitClass("player");
			local healing = heal;
			if ( BCF_GetOption("SHOWOVERHEAL") == 1 ) then
				healing = BCF_GetOverheal(target,heal);
			end
			local text;
			if (pclass == "사제") then
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				text = target..": +"..healing.."(회복의 기원)";
			else
				text = target..": +"..healing.."(회복의 기원)";
			end
			end
			BCF_DisplayMessage({
				text = text,
				fontheight = BCF_GetOption("FONTSIZE"),
				rgb = BCF_GetColor("HEAL"),
				paneltype = "crit",
				font = "Fonts\\K_Damage.TTF",
			});
		end,
		commandkey = "SHOWHEAL",		
	});

	-- 회복의 기원 : test
	BCF_Register({
		event = "CHAT_MSG_SPELL_FRIENDLYPLAYER_BUFF",
		pattern = "회복의 기원|1으로;로; (.+)의 생명력이 (%d+)만큼 회복되었습니다.",
		showmsg = 0,
		callback = function(event,msg,target,heal)
			local pclass = UnitClass("player");
			local healing = heal;
			if ( BCF_GetOption("SHOWOVERHEAL") == 1 ) then
				healing = BCF_GetOverheal(target,heal);
			end
			local text;
			if (pclass == "사제") then
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				text = target..": +"..healing.."(회복의 기원)";
			else
				text = target..": +"..healing.."(회복의 기원)";
			end
			end
			BCF_DisplayMessage({
				text = text,
				fontheight = BCF_GetOption("FONTSIZE"),
				rgb = BCF_GetColor("HEAL"),
				paneltype = "fountain",
				font = "Fonts\\K_Damage.TTF",
			});
		end,
		commandkey = "SHOWHEAL",		
	});

	-- 회복의 기원 : test
	BCF_Register({
		event = "CHAT_MSG_SPELL_FRIENDLYPLAYER_BUFF",
		pattern = "회복의 기원|1이;가; 극대화 효과를 발휘하여 (.+)의 생명력이 (%d+)만큼 회복되었습니다.",
		showmsg = 0,
		callback = function(event,msg,target,heal)
			local pclass = UnitClass("player");
			local healing = heal;
			if ( BCF_GetOption("SHOWOVERHEAL") == 1 ) then
				healing = BCF_GetOverheal(target,heal);
			end
			local text;
			if (pclass == "사제") then
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				text = target..": +"..healing.."(회복의 기원)";
			else
				text = target..": +"..healing.."(회복의 기원)";
			end
			end
			BCF_DisplayMessage({
				text = text,
				fontheight = BCF_GetOption("FONTSIZE"),
				rgb = BCF_GetColor("HEAL"),
				paneltype = "crit",
				font = "Fonts\\K_Damage.TTF",
			});
		end,
		commandkey = "SHOWHEAL",		
	});

	-- 회복의 기원 : test
	BCF_Register({
		event = "CHAT_MSG_SPELL_PET_BUFF",
		pattern = "회복의 기원|1으로;로; (.+)의 생명력이 (%d+)만큼 회복되었습니다.",
		showmsg = 0,
		callback = function(event,msg,target,heal)
			local pclass = UnitClass("player");
			local healing = heal;
			if ( BCF_GetOption("SHOWOVERHEAL") == 1 ) then
				healing = BCF_GetOverheal(target,heal);
			end
			local text;
			if (pclass == "사제") then
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				text = target..": +"..healing.."(회복의 기원)";
			else
				text = target..": +"..healing.."(회복의 기원)";
			end
			end
			BCF_DisplayMessage({
				text = text,
				fontheight = BCF_GetOption("FONTSIZE"),
				rgb = BCF_GetColor("HEAL"),
				paneltype = "fountain",
				font = "Fonts\\K_Damage.TTF",
			});
		end,
		commandkey = "SHOWHEAL",		
	});

	-- 회복의 기원 : test
	BCF_Register({
		event = "CHAT_MSG_SPELL_PET_BUFF",
		pattern = "회복의 기원|1이;가; 극대화 효과를 발휘하여 (.+)의 생명력이 (%d+)만큼 회복되었습니다.",
		showmsg = 0,
		callback = function(event,msg,target,heal)
			local pclass = UnitClass("player");
			local healing = heal;
			if ( BCF_GetOption("SHOWOVERHEAL") == 1 ) then
				healing = BCF_GetOverheal(target,heal);
			end
			local text;
			if (pclass == "사제") then
			if ( BCF_GetOption("SHOWSPELLNAME") == 1 ) then
				text = target..": +"..healing.."(회복의 기원)";
			else
				text = target..": +"..healing.."(회복의 기원)";
			end
			end
			BCF_DisplayMessage({
				text = text,
				fontheight = BCF_GetOption("FONTSIZE"),
				rgb = BCF_GetColor("HEAL"),
				paneltype = "crit",
				font = "Fonts\\K_Damage.TTF",
			});
		end,
		commandkey = "SHOWHEAL",		
	});

	-- 회복의 기원 : test
	BCF_Register({
		event = "CHAT_MSG_SPELL_PERIODIC_PARTY_BUFFS",
		pattern = "(.+)|1이;가; 회복의 기원 효과를 얻었습니다.",
		msg = "회복의 기원 => %1",
		fontheight = BCF_GetOption("FONTSIZE"),
		rgb = BCF_GetColor("HEAL"),
		paneltype = "fountain",
		font = "Fonts\\K_Damage.TTF",
		class = "사제",
		commandkey = "SHOWHEAL",		
	});

	-- 회복의 기원 : test
	BCF_Register({
		event = "CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_BUFFS",
		pattern = "(.+)|1이;가; 회복의 기원 효과를 얻었습니다.",
		msg = "회복의 기원 => %1",
		fontheight = BCF_GetOption("FONTSIZE"),
		rgb = BCF_GetColor("HEAL"),
		paneltype = "fountain",
		font = "Fonts\\K_Damage.TTF",
		class = "사제",
		commandkey = "SHOWHEAL",		
	});

	-- 회복의 기원 : test
	BCF_Register({
		event = "CHAT_MSG_SPELL_PERIODIC_PET_BUFFS",
		pattern = "(.+)|1이;가; 회복의 기원 효과를 얻었습니다.",
		msg = "회복의 기원 => %1(pet)",
		fontheight = BCF_GetOption("FONTSIZE"),
		rgb = BCF_GetColor("HEAL"),
		paneltype = "fountain",
		font = "Fonts\\K_Damage.TTF",
		class = "사제",
		commandkey = "SHOWHEAL",		
	});

	-- 회복의 기원 : test
	BCF_Register({
		event = "CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS",
		pattern = "당신은 회복의 기원 효과를 얻었습니다.",
		msg = "회복의 기원 => 자신",
		fontheight = BCF_GetOption("FONTSIZE"),
		rgb = BCF_GetColor("HEAL"),
		paneltype = "fountain",
		font = "Fonts\\K_Damage.TTF",
		class = "사제",
		commandkey = "SHOWHEAL",		
	});



-----------------------------------------------
-- /명령어 등록
-----------------------------------------------

	BCF_AddCommand("SHOWOVERHEAL",{
		title = "오버힐표시",
		cmd = {"오버힐","overheal"},
		helpmsg = "%s: 힐을 받을때/줄때 오버힐량을 표시할지 선택합니다.",
		statusmsg = " 오버힐 메세지 : %s",
		cmdfunc = function ()
			local r = BCF_GetOption("SHOWOVERHEAL");
			if ( r ~= 1 ) then
				BCF_SetOption("SHOWOVERHEAL", 1);
			else
				BCF_SetOption("SHOWOVERHEAL", 0);
			end
		end,
		default = 1,
	});

	BCF_AddCommand("SHOWGAINPOWER",{
		title = "파워 회복 메세지",
		cmd = {"파워회복","gainpower"},
		helpmsg = "%s: 마나, 분노, 기력 회복 메세지를 표시합니다.",
		statusmsg = " 파워 회복 메세지 : %s",
		cmdfunc = function ()
			local r = BCF_GetOption("SHOWGAINPOWER");
			if ( r ~= 1 ) then
				BCF_SetOption("SHOWGAINPOWER", 1);
			else
				BCF_SetOption("SHOWGAINPOWER", 0);
			end
		end,
		default = 1,
	});

	BCF_AddCommand("SHOWDOUBLEHIT",{
		title = "추가타 메세지",
		cmd = {"추가타","doublehit"},
		helpmsg = "%s: 추가타가 발동될때 메세지를 보여줍니다.",
		statusmsg = " 추가타 메세지 : %s",
		cmdfunc = function ()
			local r = BCF_GetOption("SHOWDOUBLEHIT");
			if ( r ~= 1 ) then
				BCF_SetOption("SHOWDOUBLEHIT", 1);
			else
				BCF_SetOption("SHOWDOUBLEHIT", 0);
			end
		end,
		default = 1,
	});

	BCF_AddCommand("SHOWHONOR",{
		title = "예상 명예 점수 메세지",
		cmd = {"명예점수","honor"},
		helpmsg = "%s: 명예 예상점수를 보여줍니다.",
		statusmsg = " 명예 예상 점수 : %s",
		cmdfunc = function ()
			local r = BCF_GetOption("SHOWHONOR");
			if ( r ~= 1 ) then
				BCF_SetOption("SHOWHONOR", 1);
			else
				BCF_SetOption("SHOWHONOR", 0);
			end
		end,
		default = 1,
	});

	BCF_AddCommand("SHOWREPUTATIONUP",{
		title = "평판 상승 메세지",
		cmd = {"평판상승","reputationup"},
		helpmsg = "%s: 평판이 상승할 때 메세지를 보여줍니다.",
		statusmsg = " 평판 상승 메세지 : %s",
		cmdfunc = function ()
			local r = BCF_GetOption("SHOWREPUTATIONUP");
			if ( r ~= 1 ) then
				BCF_SetOption("SHOWREPUTATIONUP", 1);
			else
				BCF_SetOption("SHOWREPUTATIONUP", 0);
			end
		end,
		default = 1,
	});

	BCF_AddCommand("SHOWREPUTATIONDOWN",{
		title = "평판 하락 메세지",
		cmd = {"평판하락","reputationdown"},
		helpmsg = "%s: 평판이 하락할 때 메세지를 보여줍니다.",
		statusmsg = " 평판 하락 메세지 : %s",
		cmdfunc = function ()
			local r = BCF_GetOption("SHOWREPUTATIONDOWN");
			if ( r ~= 1 ) then
				BCF_SetOption("SHOWREPUTATIONDOWN", 1);
			else
				BCF_SetOption("SHOWREPUTATIONDOWN", 0);
			end
		end,
		default = 1,
	});

	BCF_AddCommand("SHOWAUTOREPUTATION",{
		title = "평판 자동 표시",
		cmd = {"평판자동표시","autoreputation"},
		helpmsg = "%s: 평판이 오르거나 내려가면 자동으로 해당 평판 그래프를 표시해줍니다.",
		statusmsg = " 평판 자동 표시 : %s",
		cmdfunc = function ()
			local r = BCF_GetOption("SHOWAUTOREPUTATION");
			if ( r ~= 1 ) then
				BCF_SetOption("SHOWAUTOREPUTATION", 1);
			else
				BCF_SetOption("SHOWAUTOREPUTATION", 0);
			end
		end,
		default = 1,
	});

	BCF_AddCommand("SHOWSPELLACTIVE",{
		title = "주문/기술 활성화",
		cmd = {"주문활성화","기술활성화","spellactive"},
		helpmsg = "%s: 특정 조건에서만 사용 가능한 주문이나 기술이 활성화되었을 때 알림 메시지를 표시합니다. (예: 전사의 제압)",
		statusmsg = " 주문/기술 활성화 : %s",
		cmdfunc = function ()
			local r = BCF_GetOption("SHOWSPELLACTIVE");
			if ( r ~= 1 ) then
				BCF_SetOption("SHOWSPELLACTIVE", 1);
			else
				BCF_SetOption("SHOWSPELLACTIVE", 0);
			end
		end,
		default = 1,
	});
end

