
-- 패널 움직임 함수 테이블
BCF_ScrollFunctions = {};

-- standard scroll
BCF_ScrollFunctions["standard"] = {
	speed = 1.0,
	fadeInTime = 0.75,
	holdTime = 1.2,
	fadeOutTime = 1.1,
	range = 160,
	spacing = 15,
	paneltype = "standard",
	scroll = function (self,panel)
		local elapsed = (GetTime() - panel.startTime) * self.speed;
		local position, cur_yPos, alpha;
		local fadeInTime = self.fadeInTime;
		local holdTime = self.holdTime;
		local fadeOutTime = self.fadeOutTime;
		local movingTime = fadeInTime + holdTime + fadeOutTime;

		position = elapsed / movingTime;

		if ( elapsed < fadeInTime ) then
			alpha = (elapsed / fadeInTime);
		elseif ( elapsed < (fadeInTime + holdTime) ) then
			alpha = 1.0;
		elseif ( elapsed < (fadeInTime + holdTime + fadeOutTime) ) then
			alpha = 1.0 - ((elapsed - holdTime - fadeInTime) / fadeOutTime);
		else
			BCF_RemovePanel(panel);
			return;
		end

		if ( BCF_GetOption("DIRECTION") == 1 ) then
			cur_yPos = (self.range * position) * -1;
		else
			cur_yPos = self.range * position;
		end

		panel.alpha = alpha;
		panel.yPos = cur_yPos + panel.alter;
		panel:SetAlpha(panel.alpha);
		panel:SetPoint("CENTER", (panel.anchor or "BCF_Message_Knob"), "CENTER", panel.xPos, panel.yPos );
		return;
	end,

	updatePosition = function (self,panel)
		local yPosLowestMessage = 0;
		if ( BCF_GetOption("DIRECTION") == 1 ) then -- up
			for index, value in pairs(BCF_Objects) do
				if ( value.paneltype == panel.paneltype and value.yPos > ((panel.fontheight/2 + self.spacing) * -1) ) then
					if ( yPosLowestMessage > ((panel.fontheight/2+self.spacing) * -1) - value.yPos ) then
						yPosLowestMessage = ((panel.fontheight/2+self.spacing) * -1) - value.yPos;
					end
				end
			end
		else -- down
			for index, value in pairs(BCF_Objects) do
				if ( value.paneltype == panel.paneltype and value.yPos < (panel.fontheight/2 + self.spacing) ) then
					if ( yPosLowestMessage < (panel.fontheight/2+self.spacing) - value.yPos ) then
						yPosLowestMessage = (panel.fontheight/2+self.spacing) - value.yPos
					end
				end
			end
		end
		if ( yPosLowestMessage ~= 0 ) then
			for index, value in pairs(BCF_Objects) do
				if ( value.paneltype == panel.paneltype ) then
					value.alter = value.alter + yPosLowestMessage;
				end
			end
		end
		panel.yPos = 0;
	end,
};

-- 2배속 스크롤
BCF_ScrollFunctions["x2"] = {
	speed = 1.0,
	fadeInTime = 0.75,
	holdTime = 1.2,
	fadeOutTime = 1.1,
	range = 160,
	spacing = 15,
	paneltype = "x2",
	scroll = function (self,panel)
		local elapsed = ( GetTime() - panel.startTime ) * 2 * self.speed;
		local position, cur_yPos, alpha;
		local fadeInTime = self.fadeInTime;
		local holdTime = self.holdTime;
		local fadeOutTime = self.fadeOutTime;
		local movingTime = fadeInTime + holdTime + fadeOutTime;

		position = elapsed / movingTime;

		if ( elapsed < fadeInTime ) then
			alpha = (elapsed / fadeInTime);
		elseif ( elapsed < (fadeInTime + holdTime) ) then
			alpha = 1.0;
		elseif ( elapsed < (fadeInTime + holdTime + fadeOutTime) ) then
			alpha = 1.0 - ((elapsed - holdTime - fadeInTime) / fadeOutTime);
		else
			BCF_RemovePanel(panel);
			return;
		end

		if ( BCF_GetOption("DIRECTION") == 1 ) then
			cur_yPos = (self.range * position) * -1;
		else
			cur_yPos = self.range * position;
		end

		panel.alpha = alpha;
		panel.yPos = cur_yPos + panel.alter;
		panel:SetAlpha(panel.alpha);
		panel:SetPoint("CENTER", (panel.anchor or "BCF_Message_Knob"), "CENTER", panel.xPos, panel.yPos );
		return;
	end,
	updatePosition = function (self,panel)
		local yPosLowestMessage = 0;
		if ( BCF_GetOption("DIRECTION") == 1 ) then -- up
			for index, value in pairs(BCF_Objects) do
				if ( value.paneltype == panel.paneltype and value.yPos > ((panel.fontheight/2 + self.spacing) * -1) ) then
					if ( yPosLowestMessage > ((panel.fontheight/2+self.spacing) * -1) - value.yPos ) then
						yPosLowestMessage = ((panel.fontheight/2+self.spacing) * -1) - value.yPos;
					end
				end
			end
		else -- down
			for index, value in pairs(BCF_Objects) do
				if ( value.paneltype == panel.paneltype and value.yPos < (panel.fontheight/2 + self.spacing) ) then
					if ( yPosLowestMessage < (panel.fontheight/2+self.spacing) - value.yPos ) then
						yPosLowestMessage = (panel.fontheight/2+self.spacing) - value.yPos
					end
				end
			end
		end
		if ( yPosLowestMessage ~= 0 ) then
			for index, value in pairs(BCF_Objects) do
				if ( value.paneltype == panel.paneltype ) then
					value.alter = value.alter + yPosLowestMessage;
				end
			end
		end
		panel.yPos = 0;
	end,
};

-- sticky panel
BCF_ScrollFunctions["sticky"] = {
	fadeInTime = 0.75,
	holdTime = 1.2,
	fadeOutTime = 1.1,
	paneltype = "sticky",
	scroll = function (self,panel)
		local elapsed = GetTime() - panel.startTime;
		if ( elapsed < (self.fadeInTime + self.holdTime) ) then
			alpha = 1.0;
		elseif ( elapsed < (self.fadeInTime + self.holdTime + self.fadeOutTime) ) then
			alpha = 1.0 - ((elapsed - self.holdTime - self.fadeInTime) / self.fadeOutTime);
		else
			BCF_RemovePanel(panel);
			return;
		end
		panel.alpha = alpha;
		panel:SetAlpha(panel.alpha);
		return;
	end,

	updatePosition = function (self,panel)
		-- nothing
		panel.yPos = -70;
	end,
};

-- crit effect scroll
BCF_ScrollFunctions["crit"] = {
	speed = 1.0,
	paneltype = "crit",
	increaseTime = 0.1,
	decreaseTime = 0.1,
	holdTime = 1.25,
	fadeOutTime = 1.0,
	lastCritPanel = 1,
	xOffset = 80,
	yOffset = 30,
	panelY = 40,
	scroll = function (self,panel)
		local elapsed = GetTime() - panel.startTime;
		local alpha;
		local fontheight = panel.fontheight;
		if ( elapsed < self.increaseTime ) then
			alpha = 1.0;
			fontheight = panel.fontheight * (elapsed / self.increaseTime) * 1.5;
		elseif ( elapsed < (self.increaseTime + self.decreaseTime) ) then
			alpha = 1.0;
			fontheight = (panel.fontheight * 1.5) - ((elapsed - self.increaseTime) / self.decreaseTime * (panel.fontheight*0.5));
		elseif ( elapsed < (self.increaseTime + self.decreaseTime + self.holdTime) ) then
			alpha = 1.0;
			fontheight = panel.fontheight;
		elseif ( elapsed < (self.increaseTime + self.decreaseTime + self.holdTime + self.fadeOutTime) ) then
			alpha = 1.0 - ((elapsed - self.increaseTime - self.decreaseTime - self.holdTime) / self.fadeOutTime);
			fontheight = panel.fontheight;
		else
			BCF_RemovePanel(panel);
			return;
		end
		if fontheight <= 0 then
			fontheight = 1;
		end

		panel.alpha = alpha;
		panel:SetAlpha(panel.alpha);
		panel:SetTextHeight(fontheight);
		return;
	end,

	updatePosition = function (self,panel)
		local critcount = BCF_GetPanelCount("crit");
		if ( critcount > 0 ) then
			if ( self.lastCritPanel == 1 ) then
				panel.xPos = self.xOffset;
				panel.yPos = self.panelY - self.xOffset;
				self.lastCritPanel = 2;
			elseif ( self.lastCritPanel == 2 ) then
				panel.xPos = 0 - self.xOffset;
				panel.yPos = self.panelY + self.yOffset;
				self.lastCritPanel = 3;
			elseif ( self.lastCritPanel == 3 ) then
				panel.xPos = 0 - self.xOffset;
				panel.yPos = self.panelY - self.yOffset;
				self.lastCritPanel = 4;
			elseif ( self.lastCritPanel == 4 ) then
				panel.xPos = self.xOffset;
				panel.yPos = self.panelY + self.yOffset;
				self.lastCritPanel = 5;
			elseif ( self.lastCritPanel == 5 ) then
				panel.xPos = 0;
				panel.yPos = self.panelY;
				self.lastCritPanel = 1;
			else
				panel.xPos = self.xOffset;
				panel.yPos = self.panelY - self.yOffset;
				self.lastCritPanel = 1;
			end
		else
			panel.xPos = 0;
			panel.yPos = self.panelY;
			self.lastCritPanel = 1;
		end
		local dir = 1;
		if ( BCF_GetOption("DIRECTION") == 1 ) then
			dir = -1;
		end
		panel.yPos = panel.yPos * dir;
	end,
};

-- bcfsa effect scroll
BCF_ScrollFunctions["bcfsa"] = {
	speed = 0.6,
	paneltype = "bcfsa",
	increaseTime = 0,
	decreaseTime = 0.3,
	holdTime = 0.8,
	fadeOutTime = 0.6,
	spacing = 60,
	yOffset = 20,
	panelY = 60,
	scroll = function (self,panel)
		local elapsed = GetTime() - panel.startTime;
		local alpha;
		local fontheight = panel.fontheight;
		if ( elapsed < self.increaseTime ) then
			alpha = 1.0;
			fontheight = panel.fontheight * (elapsed / self.increaseTime) * 1;
		elseif ( elapsed < (self.increaseTime + self.decreaseTime) ) then
			alpha = 0 + ((elapsed - self.increaseTime - self.decreaseTime - self.holdTime) / self.fadeOutTime);
			fontheight = (panel.fontheight * 1.5) - (elapsed / self.decreaseTime * panel.fontheight) * 0.5;
		elseif ( elapsed < (self.increaseTime + self.decreaseTime + self.holdTime) ) then
			alpha = 1.0;
			fontheight = panel.fontheight;
		elseif ( elapsed < (self.increaseTime + self.decreaseTime + self.holdTime + self.fadeOutTime) ) then
			alpha = 1.0 - ((elapsed - self.increaseTime - self.decreaseTime - self.holdTime) / self.fadeOutTime);
			fontheight = panel.fontheight;
		else
			BCF_RemovePanel(panel);
			return;
		end
		if fontheight <= 0 then
			fontheight = 1;
		end

		panel.alpha = alpha;
		panel:SetAlpha(panel.alpha);
		panel:SetTextHeight(fontheight);
		panel:SetPoint("CENTER", (panel.anchor or "BCF_Message_Knob"), "CENTER", panel.xPos, panel.yPos );
		return;
	end,

	updatePosition = function (self,panel)
			local yLowestMessage = 0;
			for index, value in pairs(BCF_Objects) do
				if ( value.paneltype == self.paneltype ) then
					if ( value.yPos ) then
--						if ( yLowestMessage < self.spacing - value.xPos - value.alter ) then
	--					DEFAULT_CHAT_FRAME:AddMessage("test");
	--						yLowestMessage =yLowestMessage+1;
						value.yPos = value.yPos+ self.yOffset;
--						end
					end
				end
			end
		--panel.xPos = 0;
		panel.yPos = 200;
	end,
};

-- 분수모양 스크롤 from CombatText - Blizzard
BCF_ScrollFunctions["fountain"] = {
	paneltype = "fountain",
	lastXDir = 1,
	alterTime = 0.3,
	scroll = function (self,panel)
		local scrollSpeed = 1.9;
		local alpha;
		local elapsed = GetTime() - panel.startTime;
		
		if ( elapsed < 1.0 ) then
			alpha = 1.0;
		elseif ( elapsed >= 1.0 and elapsed < scrollSpeed ) then
			alpha = 1.0 - ((elapsed - 1.0) / 0.9);
		elseif ( elapsed >= scrollSpeed ) then
			BCF_RemovePanel(panel);
			return;
		end

		local radius = 160;
		local xPos = panel.xDir * ( radius * cos(90 * elapsed / scrollSpeed) ) - panel.xDir * radius;
		local yPos = radius * sin(90 * elapsed / scrollSpeed);
		
		panel.alpha = alpha;
		panel.xPos = xPos;
		panel.yPos = yPos;
		panel:SetAlpha(panel.alpha);
		panel:SetPoint("CENTER", (panel.anchor or "BCF_Message_Knob"), "CENTER", panel.xPos, panel.yPos );
		return;
	end,

	updatePosition = function (self,panel)
		if ( self.lastXDir == 1 ) then
			panel.xDir = -1;
			self.lastXDir = -1;
		else
			panel.xDir = 1;
			self.lastXDir = 1;
		end
--		BCF_Print("panel.xDir = "..panel.xDir);
--		BCF_Print("self.lastXDir = "..self.lastXDir);

		local stLowestMessage = 0;
		for index, value in pairs(BCF_Objects) do
			if ( value.paneltype == self.paneltype and value.xDir == panel.xDir ) then
				if ( value.startTime > panel.startTime - self.alterTime ) then
					if ( stLowestMessage > (panel.startTime - self.alterTime) - value.startTime ) then
						stLowestMessage = (panel.startTime + self.alterTime) - value.startTime;
					end
				end
			end
		end

		if ( stLowestMessage < 0 ) then
			for index, value in pairs(BCF_Objects) do
				if ( value.paneltype == self.paneltype and value.xDir == panel.xDir ) then
					value.startTime = value.startTime + stLowestMessage;
				end
			end
		end
		panel.xPos = 0;
		panel.yPos = 0;
	end,
};


-- 중력(?) 스크롤
BCF_ScrollFunctions["gravity"] = {

	paneltype = "gravity",
	spacing = 25,
	yOffset = 30,
	
	scroll = function (self,panel)
		local scrollSpeed = 1.9;
		local alpha;
		local elapsed = GetTime() - panel.startTime;
		
		if ( elapsed < 1.0 ) then
			alpha = 1.0;
		elseif ( elapsed >= 1.0 and elapsed < scrollSpeed ) then
			alpha = 1.0 - ((elapsed - 1.0) / 0.9);
		elseif ( elapsed >= scrollSpeed ) then
			BCF_RemovePanel(panel);
			return;
		end

		local radius = 180;
		local yPos;
		
		if ( BCF_GetOption("DIRECTION") == 1 ) then
			yPos = radius * sin(90 * elapsed / scrollSpeed);
		else
			yPos = (radius * cos(90 * elapsed / scrollSpeed)) - radius;
		end

		panel.alpha = alpha;
		panel.yPos = yPos + panel.alter;
		panel:SetAlpha(panel.alpha);
		panel:SetPoint("CENTER", (panel.anchor or "BCF_Message_Knob"), "CENTER", panel.xPos, panel.yPos );
		return;
	end,

	updatePosition = function (self,panel)
		local yLowestMessage = 0;
		if ( BCF_GetOption("DIRECTION") == 1 ) then
			for index, value in pairs(BCF_Objects) do
				if ( value.paneltype == self.paneltype ) then
					if ( value.yPos < self.spacing ) then
						if ( yLowestMessage < self.spacing - value.yPos - value.alter ) then
							yLowestMessage = self.spacing - value.yPos - value.alter;
						end
					end
				end
			end
		else
			for index, value in pairs(BCF_Objects) do
				if ( value.paneltype == self.paneltype ) then
					if ( value.yPos > (self.spacing*-1) ) then
						if ( yLowestMessage > (self.spacing*-1) + value.yPos + value.alter ) then
							yLowestMessage = (self.spacing*-1) + value.yPos + value.alter;
						end
					end
				end
			end
		end
		if ( yLowestMessage > 0 ) then
			for index, value in pairs(BCF_Objects) do
				if ( value.paneltype == self.paneltype ) then
					value.alter = value.alter + yLowestMessage;
				end
			end
		end
		--panel.xPos = 0;
		panel.yPos = 0;
	end,
};
