local Capping = Capping
local self = Capping
local L = CappingLocale

local wsgicon
local af, aftext, acarrier, aclass
local hf, hftext, hcarrier, hclass

local SetWSGCarrierAttribute
-- set alliance carrier data and text
local function SetACarrier(carrier, class)
	acarrier = carrier
	aclass = class
	aftext:SetText(format("|cff%s%s|r", self:GetClassColorHex(class), carrier or ""))
end
-- set horde carrier data and text
local function SetHCarrier(carrier, class)
	hcarrier = carrier
	hclass = class
	hftext:SetText(format("|cff%s%s|r", self:GetClassColorHex(class), carrier or ""))
end
-- stuff to do at the beginning of every wsg (but wait til after combat)
local function Bulk()
	SetACarrier()
	SetHCarrier()
	af:Show()
	hf:Show()
	SetWSGCarrierAttribute()
	self:RegisterTempEvent("CHAT_MSG_BG_SYSTEM_HORDE", "WSGFlagCarrier")
	self:RegisterTempEvent("CHAT_MSG_BG_SYSTEM_ALLIANCE", "WSGFlagCarrier")
	self:RegisterTempEvent("CHAT_MSG_BG_SYSTEM_NEUTRAL", "CheckStartTimer")
	
	-- makes sure GetBattlefieldScore will return correct values
	-- note: waiting for UPDATE_BATTLEFIELD_SCORE is just too slow (1-3 secs)
	RequestBattlefieldScoreData()
end

---------------------------
function Capping:StartWSG()
---------------------------
	-- init some data and create carrier frames
	if not af then
		wsgicon = strlower(UnitFactionGroup("player"))
		self:CheckCombat(self.CreateWSGFrame)
	end
	self:CheckCombat(Bulk)
end

-- carrier detection and setup handlers
-----------------------------------
function Capping:WSGFlagCarrier(a1)
-----------------------------------
	-- flag was picked up
	if strfind(a1, L["was picked up by (.+)!"]) then
		local _,_,name = strfind(a1, L["was picked up by (.+)!"])
		if strfind(a1, L["Horde"]) then 
			faction = 1  -- alliance carrier
		else 
			faction = 0  -- horde carrier
		end
		if faction == 1 then
			SetACarrier(name, self:GetClassByName(name, faction))
			af:SetAlpha(0.5)
		else
			SetHCarrier(name, self:GetClassByName(name, faction))
			hf:SetAlpha(0.5)
		end
		self:CheckCombat(SetWSGCarrierAttribute)
	-- flag was dropped, reset for that carrier
	elseif strfind(a1, L["dropped"]) then
		if strfind(a1, L["Horde"]) then
			SetACarrier()
		else
			SetHCarrier()
		end
		self:CheckCombat(SetWSGCarrierAttribute)
	-- flag was captured, reset all carriers
	elseif strfind(a1, L["captured the"]) then
		SetACarrier()
		SetHCarrier()
		self:CheckCombat(SetWSGCarrierAttribute)
		self:StartBar(L["Flag respawns"], 23, self:GetIconData(wsgicon, "flag"), "info1")
	end
end


-- sends basic carrier info to battleground chat
local function CarrierOnClick(this)
	if IsControlKeyDown() then 
		local faction = this.faction
		local carrier = (faction == "Horde" and hcarrier) or acarrier
		if not carrier then return end
		local text = format(L["%s's flag carrier: %s (%s)"], L[faction], carrier, (faction == "Horde" and hclass) or aclass)
		SendChatMessage(text, "BATTLEGROUND")
	end 
end
local function CreateCarrierFrame(faction)
	local f = CreateFrame("Button", "CappingTarget"..faction, UIParent, "SecureActionButtonTemplate")
	f:SetHeight(18)
	f:SetWidth(110)
	f:RegisterForClicks("LeftButtonUp")
	f:SetAttribute("type1", "macro")
	f:SetAttribute("macrotext", "")
	f.faction = faction
	f:SetScript("PostClick", CarrierOnClick)

	f.text = f:CreateFontString(nil, "BACKGROUND")
	f.text:SetFont(GameFontHighlightSmall:GetFont(), 14)
	f.text:SetJustifyH("LEFT")
	f.text:ClearAllPoints()
	f.text:SetAllPoints(f)
	f.text:SetShadowColor(0,0,0)
	f.text:SetShadowOffset(1,-1)
	self:AddFrameToHide(f)
	return f
end

-- needed so that AlwaysUpFrame is not forced to be secure
local function SetWSGCarrierPositions()
	af:SetPoint("LEFT", UIParent, "BOTTOMLEFT", AlwaysUpFrame1:GetRight() + 38, AlwaysUpFrame1:GetTop())
	hf:SetPoint("LEFT", UIParent, "BOTTOMLEFT", AlwaysUpFrame2:GetRight() + 38, AlwaysUpFrame2:GetTop())
end

-- creates the button frames on the scoreboard
local elap = 0
---------------------------------
function Capping:CreateWSGFrame()
---------------------------------
	af = CreateCarrierFrame("Alliance")
	hf = CreateCarrierFrame("Horde")

	aftext, hftext = af.text, hf.text
	af:SetScript("OnUpdate", function(this, a1)
		elap = elap + a1
		if elap < 10 then return end
		elap = 0
		RequestBattlefieldScoreData()
	end)
	
	SetWSGCarrierPositions()
end

-- props to "Fedos" and the ICU mod
-- updates the button to target by name
-- button will be slightly transparent if button cannot update (in combat)
function SetWSGCarrierAttribute()
	af:SetAlpha(1)
	hf:SetAlpha(1)
	af:SetAttribute("macrotext", acarrier and format("/target %s", acarrier) or "")
	hf:SetAttribute("macrotext", hcarrier and format("/target %s", hcarrier) or "")

	SetWSGCarrierPositions()
end
