﻿--{{{ Libraries

local L = AceLibrary("AceLocale-2.2"):new("GridStatusHealer")
local Aura = AceLibrary("SpecialEvents-Aura-2.0")
local Banzai = AceLibrary("Banzai-1.1")
local RL = AceLibrary("Roster-2.1")
local Threat = AceLibrary("Threat-1.0")
if AceLibrary:HasInstance("IncomingHealsLib-1.0") then
	local IHL = AceLibrary("IncomingHealsLib-1.0")
end
if AceLibrary:HasInstance("HealComm-2.0") then
	local HC = AceLibrary("HealComm-2.0")
end

--}}}


GridStatusHealer = GridStatus:NewModule("GridStatusHealer")
GridStatusHealer.menuName = L["Healer"]


--{{{ AceDB defaults

GridStatusHealer.defaultDB = {
	debug = false,
	alert_firstPriority = {
		text = L["First"],
		enable = true,
		color = { r = 1, g = .2, b = 0, a = 1 },
		priority = 99,
		range = true,
	},
	alert_secondPriority = {
		text = L["Second"],
		enable = true,
		color = { r = .5, g = .4, b = 0, a = .7 },
		priority = 99,
		range = true,
	},
}

--}}}


--{{{ Options

GridStatusHealer.options = false

--}}}


--{{{ locals

local firstPriorityUnit = nil
local firstPriorityValue = 998
local secondPriorityUnit = nil
local secondPriorityValue = 999

local playerClass = select(2, UnitClass("player"))

local classModifiers = {
	["PRIEST"]   = { ["WARRIOR"] =  0, ["MAGE"] =  5, ["PRIEST"] =  5, ["DRUID"] =  5, ["PALADIN"] = 10, ["SHAMAN"] = 10, ["WARLOCK"] = 10, ["HUNTER"] = 10, ["ROGUE"] = 10 },
	["DRUID"]    = { ["WARRIOR"] =  5, ["MAGE"] = 10, ["PRIEST"] = 10, ["DRUID"] =  5, ["PALADIN"] =  0, ["SHAMAN"] =  5, ["WARLOCK"] =  5, ["HUNTER"] =  5, ["ROGUE"] = 10 },
	["PALADIN"]  = { ["WARRIOR"] =  5, ["MAGE"] = 10, ["PRIEST"] =  0, ["DRUID"] = 10, ["PALADIN"] =  5, ["SHAMAN"] =  5, ["WARLOCK"] = 10, ["HUNTER"] = 10, ["ROGUE"] =  5 },
	["SHAMAN"]   = { ["WARRIOR"] =  5, ["MAGE"] = 10, ["PRIEST"] = 10, ["DRUID"] = 10, ["PALADIN"] =  5, ["SHAMAN"] =  5, ["WARLOCK"] = 8, ["HUNTER"] = 10, ["ROGUE"] =  0 },
}

local antiHealingDebuffs = {
	[L["Mortal Strike"]] = true,           -- -50% healing, warriors
	[L["Blood Fury"]] = true,              -- -50% healing, orc racial
	[L["Brood Affliction: Green"]] = true, -- -50% healing, Chromaggus, BWL
	[L["Mortal Cleave"]] = true,           -- -50% healing, High Priest Thekal, ZG
	[L["Gehennas\' Curse"]] = true,        -- -75% healing, Gehennas, MC
	[L["Veil of Shadow"]] = true,          -- -75% healing, Nefarian, BWL
	[L["Necrotic Poison"]] = true,         -- -90% healing, Maexxna, Naxx
}

local healingPreventionDebuffs = {
	[L["Enfeeble"]] = true,                -- -100% healing, Malchezaar, Kara
	[L["Aura of Suffering"]] = true,       -- -100% healing, Essence of Souls, BT
}

local hots = {
	[L["Renew"]] = true,
	[L["Rejuvenation"]] = true,
	[L["Regrowth"]] = true,
	[L["Lifebloom"]] = true,
}

local subgroupModifier = {
	[1] = 0,
	[2] = 0,
	[3] = 0,
	[4] = 0,
	[5] = 0,
	[6] = 0,
	[7] = 0,
	[8] = 0,
}

--}}}


function GridStatusHealer:OnInitialize()
	self.super.OnInitialize(self)
	self:RegisterStatus("alert_firstPriority", L["Healer: First priority"], nil, true)
	self:RegisterStatus("alert_secondPriority", L["Healer: Second priority"], nil, true)
end

function GridStatusHealer:OnEnable()
	self:RegisterEvent("UNIT_HEALTH", "UpdateUnitPriority")
	self:RegisterEvent("Banzai_UnitGainedAggro", "UpdateUnitPriority")
	self:RegisterEvent("Banzai_UnitLostAggro", "UpdateUnitPriority")
	self:RegisterEvent("SpecialEvents_UnitBuffGained", "UnitAuraChanged")
	self:RegisterEvent("SpecialEvents_UnitBuffLost", "UnitAuraChanged")
	self:RegisterEvent("SpecialEvents_UnitDebuffGained", "UnitAuraChanged")
	self:RegisterEvent("SpecialEvents_UnitDebuffLost", "UnitAuraChanged")
	self:RegisterEvent("Grid_StatusGained", "UnitStatusChanged")
	self:RegisterEvent("Grid_StatusLost", "UnitStatusChanged")
	self:RegisterEvent("RosterLib_RosterUpdated", "UpdateSubgroupModifiers")
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "UpdateSubgroupModifiers")
	if IHL then
	    self:RegisterEvent("IncomingHealsLib_HealStart", "IncomingHealsUpdate");
	    self:RegisterEvent("IncomingHealsLib_HealCancel", "IncomingHealsUpdate");
	    self:RegisterEvent("IncomingHealsLib_HealEnd", "IncomingHealsUpdate");
	    self:RegisterEvent("IncomingHealsLib_HotEnd", "IncomingHealsUpdate");
	    self:RegisterEvent("IncomingHealsLib_HotEnd", "IncomingHealsUpdate");
	    self:RegisterEvent("IncomingHealsLib_Swiftmend", "IncomingHealsUpdate");
	end
	if VisualHeal then
		VisualHeal.HealComm.RegisterCallback(self, "HealComm_DirectHealStart");
		VisualHeal.HealComm.RegisterCallback(self, "HealComm_DirectHealStop");
		VisualHeal.HealComm.RegisterCallback(self, "HealComm_DirectHealDelayed");
	elseif HC2 then
	    self:RegisterEvent("HealComm_UnitStart", "HealCommUpdate");
	end
end

function GridStatusHealer:UnitIsMainTank(unitid)
	local mainTankTable = nil
	if oRA and oRA.maintanktable then
		mainTankTable = oRA.maintanktable
	elseif CT_RATarget and CT_RATarget.MainTanks then
		mainTankTable = CT_RATarget.MainTanks
	end
	if mainTankTable then
		for key, val in pairs(oRA.maintanktable) do
			if (UnitName(unitid) == val) then
				return true
			end
		end
	end
	return nil
end

function GridStatusHealer:GetHighThreatTarget()
	local maxThreat = 0
	local highThreatTarget = nil
	local threat = 0
	local mainTankTable = nil
	if UnitExists("target") and UnitIsEnemy("player", "target") and (not UnitIsPlayer("target")) then
		threat = Threat:GetMaxThreatOnTarget("target")
		if threat > maxThreat then
			highThreatTarget = "target"
			maxThreat = threat
		end
	elseif UnitExists("target") and UnitExists("targettarget") and UnitIsEnemy("player", "targettarget") and (not UnitIsPlayer("targettarget")) then
		threat = Threat:GetMaxThreatOnTarget("targettarget")
		if threat > maxThreat then
			highThreatTarget = "targettarget"
			maxThreat = threat
		end
	end
	if oRA and oRA.maintanktable then
		mainTankTable = oRA.maintanktable
	elseif CT_RATarget and CT_RATarget.MainTanks then
		mainTankTable = CT_RATarget.MainTanks
	end
	if mainTankTable then
		for key, val in pairs(mainTankTable) do
			local threat = 0
			local target = nil
			if UnitExists(RL:GetUnitIDFromName(val)) and UnitExists(RL:GetUnitIDFromName(val).."target") then
				target = RL:GetUnitIDFromName(val).."target"
			end
			local targettarget = nil
			if target and UnitExists(RL:GetUnitIDFromName(val).."targettarget") then
				target = RL:GetUnitIDFromName(val).."targettarget"
			end
			if target and UnitIsEnemy("player", target) and (not UnitIsPlayer(target)) then
				threat = Threat:GetMaxThreatOnTarget(target)
				if threat > maxThreat then
					highThreatTarget = target
					maxThreat = threat
				end
			elseif target and targettarget and UnitIsEnemy("player", targettarget) and (not UnitIsPlayer(targettarget)) then
				threat = Threat:GetMaxThreatOnTarget(targettarget)
				if threat > maxThreat then
					highThreatTarget = targettarget
					maxThreat = threat
				end
			end
		end
	end
	return highThreatTarget
end

function GridStatusHealer:GetModifier(unitid)
	local m = 0

	-- Can't heal
	for debuff in pairs(healingPreventionDebuffs) do
		if Aura:UnitHasDebuff(unitid, debuff) then
			return 999
		end
	end

	-- RAISE PRIORITY
	-- Has aggro
	if Banzai:GetUnitAggroByUnitId(unitid) then
		m = m - 5
	end
	-- Has aggro & anti-healing debuff
	for debuff in pairs(antiHealingDebuffs) do
		if Banzai:GetUnitAggroByUnitId(unitid) and Aura:UnitHasDebuff(unitid, debuff) then
			m = m - 5
		elseif Aura:UnitHasDebuff(unitid, debuff) then
			m = m + 5
		end
	end
	-- Is main tank
	if UnitInRaid("player") and self:UnitIsMainTank(unitid) then
		m = m - 10
	end
	-- Has high threat
	target = self:GetHighThreatTarget()
	if target then
		threatMax = Threat:GetMaxThreatOnTarget(UnitName(target))
		threat = Threat:GetThreat(UnitName(unitid), target)
		if threat >= (threatMax * .95) then
			m = m - 4
		elseif threat >= (threatMax * .9) then
			m = m - 3
		elseif threat >= (threatMax * .8) then
			m = m - 2
		end
	end
	-- Is focus unit
	if UnitIsUnit(unitid, "focus") then
		m = m - 5
	end
	-- Has conflagration debuff (from Razorgore, 3000 dmg over 10sec)
	if Aura:UnitHasDebuff(unitid, L["Conflagration"]) then
		m = m - 5
	end
	-- Has WSG flag
	if Aura:UnitHasBuff(unitid, L["Warsong Flag"]) or Aura:UnitHasBuff(unitid, L["Silverwing Flag"]) then
		m = m - 5
	end

	-- LOWER PRIORITY
	-- Has incoming heal
	local healamount = 0
	
	if VisualHeal then
		healamount = VisualHeal.HealComm.UnitIncomingHealGet(UnitName(unitid), GetTime() + 3000)
	elseif HC then
		healamount = HC:GetIncomingHeal(UnitName(unitid), GetTime() + 3000)
	end
	if IHL and (IHL:GetTargetIncomingHealInfo(UnitName(unitid)) > healamount) then
		healamount = IHL:GetTargetIncomingHealInfo(UnitName(unitid))
	end
	if healamount > 0 then
		m = m + (healamount / UnitHealthMax(unitid) / 2)
	elseif GridStatus:IsStatusRegistered("alert_heals") and GridStatus:GetCachedStatus(UnitName(unitid), "alert_heals") then
		m = m + 10
	end
	-- Has heal over time
	local hottick = 0
	if IHL then
		hottick = IHL:GetTargetRunningHotInfo("player")
	end
	if hottick > 0 then
		m = m + (hottick / UnitHealthMax(unitid) / 2)
	else
		for buff in pairs(hots) do
			local buffIndex, applications = Aura:UnitHasBuff(unitid, buff)
			if buffIndex then
				m = m + (2 * applications)
			end
		end
	end
	-- Has prayer of mending
	if Aura:UnitHasBuff(unitid, L["Prayer of Mending"]) and ((UnitHealth(unitid) / UnitHealthMax(unitid)) > .3) then
		m = m + 5
	end
	-- Has shield buff
	if Aura:UnitHasBuff(unitid, L["Power Word: Shield"]) then
		m = m + 5
	end
	-- Is out of mana
	if (UnitPowerType(unitid) == 0) then
		local mana = UnitMana(unitid) / UnitManaMax(unitid)
		if (mana < 0.10) then
			m = m + 10
		elseif (mana < 0.20) then
			m = m + 5
		end
	end
	-- Is AFK
	if UnitIsAFK(unitid) then
		m = m + 20
	end
	-- Is pet
	if (UnitPlayerOrPetInRaid(unitid) or UnitPlayerOrPetInParty(unitid)) and (not UnitInRaid(unitid)) and (not UnitInParty(unitid)) then
		m = m + 5
	end
	-- Has burning adrenaline (from Vaelastrasz, 10 seconds until dead)
	if Aura:UnitHasDebuff(unitid, L["Burning Adrenaline"]) then
		m = m + 15
	end
	-- If PvE raid, de-prioritize healing certain classes based on what healing class you are
	if (select(2, IsInInstance()) == "raid") and (GetNumRaidMembers() > 10) and ((playerClass == "PRIEST") or (playerClass == "PALADIN") or (playerClass == "DRUID") or (playerClass == "SHAMAN")) then
		local unitClass = select(2, UnitClass(unitid))
		m = m + classModifiers[playerClass][unitClass]
	end
	-- If PvE raid, modify priority depending on how many healers, main tanks, & melee classes are in each subgroup
	if (select(2, IsInInstance()) == "raid") and (GetNumRaidMembers() > 5) then
		local u = RL:GetUnitObjectFromUnit(unitid)
		if u then
			m = m + subgroupModifier[u.subgroup]
		end
	end
	return m
end

function GridStatusHealer:UnitDoesntNeedPriority(unitid)
	if UnitIsDeadOrGhost(unitid) or (not UnitIsConnected(unitid)) or (GridStatus:IsStatusRegistered("alert_range_oor") and GridStatus:GetCachedStatus(UnitName(unitid), "alert_range_oor"))
			or (not UnitIsFriend(unitid, "player")) or (not UnitAffectingCombat(unitid)) or (((UnitHealth(unitid) / UnitHealthMax(unitid)) > .9) and (not Banzai:GetUnitAggroByUnitId(unitid))) then
		return true
	else
		return nil
	end
end

function GridStatusHealer:UpdateUnitPriority(unitid)
	-- If you are not grouped with the unit then abort
	if (not UnitPlayerOrPetInRaid(unitid)) and (not UnitPlayerOrPetInParty(unitid)) then
		return
	end
	-- Check to see if current priorities should stay priorities
	if secondPriorityUnit and self:UnitDoesntNeedPriority(secondPriorityUnit) then
		self.core:SendStatusLost(UnitName(secondPriorityUnit), "alert_secondPriority")
		secondPriorityUnit = nil
		secondPriorityValue = 999
	end
	local firstPrioritySettings = self.db.profile.alert_firstPriority
	if firstPriorityUnit and self:UnitDoesntNeedPriority(firstPriorityUnit) then
		self.core:SendStatusLost(UnitName(firstPriorityUnit), "alert_firstPriority")
		if secondPriorityUnit then
			self.core:SendStatusLost(UnitName(secondPriorityUnit), "alert_secondPriority")
			self.core:SendStatusGained(UnitName(secondPriorityUnit), "alert_firstPriority",
				firstPrioritySettings.priority,
				(firstPrioritySettings.range and 40),
				firstPrioritySettings.color,
				firstPrioritySettings.text,
				nil,
				nil,
				firstPrioritySettings.icon)
			firstPriorityUnit = secondPriorityUnit
			firstPriorityValue = secondPriorityValue
			secondPriorityUnit = nil
			secondPriorityValue = 999
		else
			firstPriorityUnit = nil
			firstPriorityValue = 998
		end
	end
	-- If it isn't appropriate to prioritize unit then abort
	if self:UnitDoesntNeedPriority(unitid) then
		return
	end
	-- Now check to see if this unit should be a priority
	local unitValue = floor(UnitHealth(unitid)/UnitHealthMax(unitid)*100) + self:GetModifier(unitid)
	local secondPrioritySettings = self.db.profile.alert_secondPriority
	if (unitValue < firstPriorityValue) and ((not firstPriorityUnit) or (not UnitIsUnit(unitid, firstPriorityUnit))) then
		if firstPriorityUnit then
			self.core:SendStatusLost(UnitName(firstPriorityUnit), "alert_firstPriority")
			if secondPriorityUnit then
				self.core:SendStatusLost(UnitName(secondPriorityUnit), "alert_secondPriority")
			end
			self.core:SendStatusGained(UnitName(firstPriorityUnit), "alert_secondPriority",
				secondPrioritySettings.priority,
				(secondPrioritySettings.range and 40),
				secondPrioritySettings.color,
				secondPrioritySettings.text,
				nil,
				nil,
				secondPrioritySettings.icon)
			secondPriorityUnit = firstPriorityUnit
			secondPriorityValue = firstPriorityValue
		end
		self.core:SendStatusGained(UnitName(unitid), "alert_firstPriority",
			firstPrioritySettings.priority,
			(firstPrioritySettings.range and 40),
			firstPrioritySettings.color,
			firstPrioritySettings.text,
			nil,
			nil,
			firstPrioritySettings.icon)
		firstPriorityUnit = unitid
		firstPriorityValue = unitValue
	elseif firstPriorityUnit and UnitIsUnit(unitid, firstPriorityUnit) and (unitValue ~= firstPriorityValue) then
		firstPriorityValue = unitValue
	elseif (unitValue < secondPriorityValue) and ((not secondPriorityUnit) or (not UnitIsUnit(unitid, secondPriorityUnit))) then
		if secondPriorityUnit then
			self.core:SendStatusLost(UnitName(secondPriorityUnit), "alert_secondPriority")
		end
		self.core:SendStatusGained(UnitName(unitid), "alert_secondPriority",
			secondPrioritySettings.priority,
			(secondPrioritySettings.range and 40),
			secondPrioritySettings.color,
			secondPrioritySettings.text,
			nil,
			nil,
			secondPrioritySettings.icon)
		secondPriorityUnit = unitid
		secondPriorityValue = unitValue
	elseif secondPriorityUnit and UnitIsUnit(unitid, secondPriorityUnit) and (unitValue ~= secondPriorityValue) then
		secondPriorityValue = unitValue
	end
end

function GridStatusHealer:UnitAuraChanged(unitid, aura)
	if (UnitPlayerOrPetInRaid(unitid) or UnitPlayerOrPetInParty(unitid)) then
		local needsUpdate = nil
		if (aura == L["Warsong Flag"]) or (aura == L["Silverwing Flag"]) or (aura == L["Conflagration"]) or (aura == L["Power Word: Shield"]) or (aura == L["Prayer of Mending"]) or (aura == L["Burning Adrenaline"]) then
			needsUpdate = true
		end
		if (not needsUpdate) then
			for debuff in pairs(antiHealingDebuffs) do
				if aura == debuff then
					needsUpdate = true
					break
				end
			end
			for debuff in pairs(healingPreventionDebuffs) do
				if aura == debuff then
					needsUpdate = true
					break
				end
			end
		end
		if needsUpdate then
			self:UpdateUnitPriority(unitid)
		end
	end
end

function GridStatusHealer:UnitStatusChanged(name, status)
	if ((status == "alert_heals") or (status == "alert_range_oor")) and (UnitPlayerOrPetInRaid(unitid) or UnitPlayerOrPetInParty(unitid)) then
		self:UpdateUnitPriority(RL:GetUnitIDFromName(name))
	end
end

function GridStatusHealer:IncomingHealsUpdate(target, source)
	self:UpdateUnitPriority(RL:GetUnitIDFromName(target))
end

function GridStatusHealer:HealCommUpdate(healer, target)
	self:UpdateUnitPriority(RL:GetUnitIDFromName(target))
end

function GridStatusHealer:HealComm_DirectHealStart(e, healer, target)
	self:UpdateUnitPriority(RL:GetUnitIDFromName(target))
end

function GridStatusHealer:HealComm_DirectHealStop(e, healer, target)
	self:UpdateUnitPriority(RL:GetUnitIDFromName(target))
end

function GridStatusHealer:HealComm_DirectHealDelayed(e, healer, target)
	self:UpdateUnitPriority(RL:GetUnitIDFromName(target))
end

function GridStatusHealer:UpdateSubgroupModifiers()
	local i
	for i = 1, 8 do
		subgroupModifier[i] = 0
	end
	if (select(2, IsInInstance()) == "raid") and (GetNumRaidMembers() > 5) then
		local u
		for u in RL:IterateRoster(false) do
			local unitClass = select(2, UnitClass(u.unitid))
			if self:UnitIsMainTank(u.unitid) then
				subgroupModifier[u.subgroup] = subgroupModifier[u.subgroup] - 5
			elseif (unitClass == "WARRIOR") or (unitClass == "ROGUE") then
				subgroupModifier[u.subgroup] = subgroupModifier[u.subgroup] - 2
			elseif ((unitClass == "PRIEST") or (unitClass == "DRUID") or (unitClass == "PALADIN") or (unitClass == "SHAMAN")) and (not UnitIsUnit(u.unitid, "player")) then
				subgroupModifier[u.subgroup] = subgroupModifier[u.subgroup] + 10
			end
		end
	end
end
