----------------------------------------------------------------
--	Locals
----------------------------------------------------------------
local Aura = AceLibrary("SpecialEvents-Aura-2.0")

local L
if GRIDSTATUSHEALINGREDUCED_LOCALS then
	L = setmetatable(GRIDSTATUSHEALINGREDUCED_LOCALS, {
		__index = function(self, key)
			DEFAULT_CHAT_FRAME:AddMessage("Attention! GridStatusHealingReduced is missing a translation for \""..key.."\".")
			rawset(self, key, key)
			return key
		end
	})
	GRIDSTATUSHEALINGREDUCED_LOCALS = nil
else
	L = setmetatable({}, {
		__index = function(self, key)
			rawset(self, key, key)
			return key
		end
	})
end

----------------------------------------------------------------
--	Debuffs
----------------------------------------------------------------
local healingReductionBuffs = {
	[L["Dampen Magic"]] = true,					-- -20 to -500 healing
}

local healingReductionDebuffs = {
	[L["Arcing Smash"]] = true,
	[L["Blood Fury"]] = true,
	[L["Brood Affliction: Green"]] = true,
	[L["Curse of the Bleeding Hollow"]] = true,
	[L["Curse of the Deadwood"]] = true,
	[L["Deathblow"]] = true,
	[L["Ebon Poison"]] = true,
	[L["Filet"]] = true,
	[L["Gehennas' Curse"]] = true,
	[L["Hex of Weakness"]] = true,
	[L["Magma-Thrower's Curse"]] = true,
	[L["Mortal Cleave"]] = true,
	[L["Mortal Strike"]] = true,
	[L["Mortal Wound"]] = true,
	[L["Necrotic Poison"]] = true,
	[L["Solar Strike"]] = true,
	[L["Soul Strike"]] = true,
	[L["Touch of the Forgotten"]] = true,
	[L["Veil of Shadow"]] = true,
	[L["Wound Poison"]] = true,
}

local healingPreventionDebuffs = {
	[L["Aura of Suffering"]] = true,
	[L["Enfeeble"]] = true,
}

----------------------------------------------------------------
--	Module
----------------------------------------------------------------
GridStatusHealingReduced = GridStatus:NewModule("GridStatusHealingReduced")

local mod = GridStatusHealingReduced
mod.menuName = L["Healing Reduced"]
mod.options = false
mod.defaultDB = {
	debug = false,
	alert_healingReduced = {
		text = L["Healing Reduced"],
		enable = true,
		color = { r = 0.8, g = 0.4, b = 0.8, a = 1 },
		priority = 98,
		range = true,
	},
	alert_healingPrevented = {
		text = L["Healing Prevented"],
		enable = true,
		color = { r = 0.8, g = 0.4, b = 0.8, a = 1 },
		priority = 99,
		range = true,
	},
}

function mod:OnInitialize()
	self.super.OnInitialize(self)
	self:RegisterStatus("alert_healingReduced", L["Healing Reduced"], nil, true)
	self:RegisterStatus("alert_healingPrevented", L["Healing Prevented"], nil, true)
end

function mod:OnEnable()
	self:RegisterEvent("SpecialEvents_UnitDebuffGained", "UnitAuraChanged")
	self:RegisterEvent("SpecialEvents_UnitDebuffLost", "UnitAuraChanged")
end

function mod:UnitAuraChanged(unitid)
	local d = 0
	for debuff in pairs(healingPreventionDebuffs) do
		if Aura:UnitHasDebuff(unitid, debuff) then
			d = d + 1
		end
	end
	if d > 0 then
		local settings = self.db.profile.alert_healingPrevented
		self.core:SendStatusGained(UnitName(unitid), "alert_healingPrevented", settings.priority, (settings.range and 40), settings.color, settings.text)
		return
	else
		self.core:SendStatusLost(UnitName(unitid), "alert_healingPrevented")
	end

	d = 0
	for debuff in pairs(healingReductionDebuffs) do
		if Aura:UnitHasDebuff(unitid, debuff) then
			d = d + 1
		end
	end
	if d > 0 then
		local settings = self.db.profile.alert_healingReduced
		self.core:SendStatusGained(UnitName(unitid), "alert_healingReduced", settings.priority, (settings.range and 40), settings.color, settings.text)
	else
		self.core:SendStatusLost(UnitName(unitid), "alert_healingReduced")
	end
end