﻿--[[
	Name: GridFadedBar
	Revision: $Revision: 58073 $
	Developed by: Julith @ Perenolde-EU (billgatesgut@web.de)
	Description: Adds another bar below the normal healthbar in Grid.

	Initially, I butchered the GridManaBars addon and used it as base for this.
	Credits to the creator of GridManaBars and other Grid-modules for guideline.
]]

local L = AceLibrary("AceLocale-2.2"):new("GridFadedBar")
local SML = AceLibrary:HasInstance("SharedMedia-1.0") and AceLibrary("SharedMedia-1.0") or nil

L:RegisterTranslations("enUS", function() return {
	["Faded Bar"] = true,
    ["Fadedbar intensity"] = true,
    ["Sets the intensity of the faded bar."] = true,
} end)

L:RegisterTranslations("koKR", function() return {
	["Faded Bar"] = "임시 바",
    ["Fadedbar intensity"] = "임시바 명도",
    ["Sets the intensity of the faded bar."] = "임시바의 명도를 설정합니다.",
} end)

local fbDB = AceLibrary("AceAddon-2.0"):new("AceDB-2.0")
fbDB:RegisterDB("FadedBarDB", "FadedBarDBPC")
fbDB:RegisterDefaults("profile", {
    intensity = 0.6,
} )

local indicators = GridFrame.frameClass.prototype.indicators
table.insert(indicators, { type = "fadedBar",    order = 15,  name = L["Faded Bar"] })

local statusmap = GridFrame.db.profile.statusmap
if ( not statusmap["fadedBar"] ) then
	statusmap["fadedBar"] = { unitIncomingHealth = true }
end

local fb_options = {
				["fadedbar"] = {
                    type = 'range',
                    name = L["Fadedbar intensity"],
                    desc = L["Sets the intensity of the faded bar."],
                    min = 0.0,
                    max = 1.0,
                    step = 0.01,
                    bigStep = 0.025,
                    get = function ()
                              return fbDB.db.profile.intensity
                          end,
                    set = function (v)
                              fbDB.db.profile.intensity = v
                              GridFrame:WithAllFrames(function (f) f:UpdateFadedBarColor() end)
                          end,
                }
}

table.insert(GridFrame.options.args["advanced"].args, fb_options["fadedbar"])

local AceOO = AceLibrary("AceOO-2.0")
local FadedBarGridFrameClass = AceOO.Class(GridFrame.frameClass)

local _frameClass = nil
if ( not _frameClass ) then
	_frameClass = GridFrame.frameClass
	GridFrame.frameClass = FadedBarGridFrameClass
end

function FadedBarGridFrameClass.prototype:CreateFrames()

	FadedBarGridFrameClass.super.prototype.CreateFrames(self)

	local f = self.frame

    local texture = SML and SML:Fetch("statusbar", GridFrame.db.profile.texture) or "Interface\\Addons\\Grid\\gradient32x32"

    f.FadedBar = CreateFrame("StatusBar", nil, f)
    f.FadedBar:SetStatusBarTexture(texture)
    f.FadedBar:SetMinMaxValues(0,100)
    f.FadedBar:SetValue(0)
    f.FadedBar:SetFrameLevel(3)
    f.FadedBar:SetOrientation("VERTICAL")
    f.FadedBar:SetPoint("CENTER", f.Bar, "CENTER")

    f.FadedBar:SetWidth(f.Bar:GetWidth())
    f.FadedBar:SetHeight(f.Bar:GetHeight())

    self:PlaceIndicators()

    self:UpdateFadedBarColor()

    f.Bar:SetScript("OnSizeChanged", function() if not self.frame or not self.frame.FadedBar then return end
                                                self.frame.FadedBar:SetWidth(self.frame.Bar:GetWidth())
                                                self.frame.FadedBar:SetHeight(self.frame.Bar:GetHeight())
                                    end);

end

function FadedBarGridFrameClass.prototype:SetFrameTexture(texture)
    FadedBarGridFrameClass.super.prototype.SetFrameTexture(self, texture)

    if not self.frame.FadedBar then return end

    self.frame.FadedBar:SetStatusBarTexture(texture)
end

function FadedBarGridFrameClass.prototype:PlaceIndicators()

    FadedBarGridFrameClass.super.prototype.PlaceIndicators(self)

    if not self.frame.FadedBar then return end

	local f = self.frame

	if self.orientation == "HORIZONTAL" then
		f.FadedBar:SetOrientation("HORIZONTAL")
	else
		f.FadedBar:SetOrientation("VERTICAL")
	end
end


function FadedBarGridFrameClass.prototype:SetBarColor(r, g, b, a)
    FadedBarGridFrameClass.super.prototype.SetBarColor(self, r, g, b, a)

    if not self.active then return end
    self:UpdateFadedBarColor()
end

function FadedBarGridFrameClass.prototype:UpdateFadedBarColor()
    if not self.frame.FadedBar then return end
    if GridFrame.db.profile.invertBarColor then
        local r, g, b, a = self.frame.Bar:GetStatusBarColor()
        self.frame.FadedBar:SetStatusBarColor(r * fbDB.db.profile.intensity,
                                          g * fbDB.db.profile.intensity,
                                          b * fbDB.db.profile.intensity,
                                          a)
   
    else
        self.frame.Bar:SetStatusBarColor(0.0, 0.0, 0.0, 1.0 - (0.2 / (1.0 - fbDB.db.profile.intensity * 0.8)) )
        self.frame.FadedBar:SetStatusBarColor(0.0, 0.0, 0.0, fbDB.db.profile.intensity * 0.8)
    end
    
end


function FadedBarGridFrameClass.prototype:SetIndicator(indicator, color, text, value, maxValue, texture)

	FadedBarGridFrameClass.super.prototype.SetIndicator(self,indicator, color, text, value, maxValue, texture)

    if not self.frame.FadedBar then return end

	if indicator == "fadedBar" then
        self.active = true
        if value and maxValue then
            if maxValue <= 0 or value <= 0 or value > maxValue then
                maxValue = 100
                value = 100
            end

			self.frame.FadedBar:SetValue(value/maxValue*100)
		end
    end
end

function FadedBarGridFrameClass.prototype:ClearIndicator(indicator)
    FadedBarGridFrameClass.super.prototype.ClearIndicator(self, indicator)
    
    if indicator == "fadedBar" then
        self.active = false
        self.frame.FadedBar:SetValue(0)
        if not GridFrame.db.profile.invertBarColor then
            self.frame.Bar:SetStatusBarColor(0.0, 0.0, 0.0, 0.8)
        end
    end
end
