﻿--[[
	Name: GridStatusIncomingHealth
	Revision: $Revision: 58073 $
	Developed by: Julith @ Perenolde-EU (billgatesgut@web.de)
	Description: IncomingHealsLib module for Grid

	Initially, I butchered the GridStatusIncomingHeals addon and used it as base for this.
    Added some lines from GridStatusHealth to.
	Credits to the creator of GridStatusIncomingHeals and other Grid-modules for guideline.
]]

--{{{ Libraries

local L = AceLibrary("AceLocale-2.2"):new("GridStatusIncomingHealth")
local RL = AceLibrary("Roster-2.1")
local IHL = AceLibrary("IncomingHealsLib-1.0")

--}}}

L:RegisterTranslations("enUS", function()
	return {
		["IncomingHealth"] = true,
		["IH:Health"] = "Incoming Health",
	}
end)

L:RegisterTranslations("koKR", function()
	return {
		["IncomingHealth"] = "받는 생명력",
		["IH:Health"] = "IH:생명력",
	}
end)

GridStatusIncomingHealth = GridStatus:NewModule("GridStatusIncomingHealth")
GridStatusIncomingHealth.menuName = L["IncomingHealth"]


--{{{ AceDB defaults

GridStatusIncomingHealth.defaultDB = {
	unitIncomingHealth = {
		text = "Health",
		enable = true,
		priority = 90,
		range = false,
		color = { r = 0, g = 1, b = 0, a = 1 },
		customText = "",
	},
}

--}}}

--{{{ Options
local amountOptions = {

}
--}}}

function GridStatusIncomingHealth:OnInitialize()
	self.super.OnInitialize(self)
	self:RegisterStatus("unitIncomingHealth", L["IH:Health"], amountOptions, true)
end

function GridStatusIncomingHealth:OnEnable()
	self:RegisterEvent("IncomingHealsLib_HealStart", "UpdateIncoming")
	self:RegisterEvent("IncomingHealsLib_HealCancel", "UpdateIncoming")
	self:RegisterEvent("IncomingHealsLib_HealEnd", "HealEnd")
    self:RegisterEvent("IncomingHealsLib_HotStart", "UpdateIncoming")
    self:RegisterEvent("IncomingHealsLib_HotEnd", "HealEnd")
    self:RegisterEvent("UNIT_HEALTH", "UpdateHealth")
	self:RegisterEvent("UNIT_MAXHEALTH", "UpdateHealth")
    self:RegisterEvent("Grid_UnitJoined")
	self:RegisterEvent("Grid_UnitChanged")
    self:RegisterEvent("PLAYER_ENTERING_WORLD", "UpdateAllUnits")
end

function GridStatusIncomingHealth:HealEnd(target, source)
    local _,_,name = string.find(target, "(.+)%-+.*")
    if not name then
        name = target
    end
    local id = RL:GetUnitIDFromName(name)
    if not id then return end
    if UnitHealth(id) == UnitHealthMax(id) then
        self:Update(id)
    end
end

function GridStatusIncomingHealth:UpdateAllUnits()
	local name, status, statusTbl

	for name, status, statusTbl in self.core:CachedStatusIterator("unitIncomingHealth") do
		self:Grid_UnitJoined(name)
	end
end

function GridStatusIncomingHealth:Grid_UnitJoined(name, unitid)
	if unitid then
		self:Update(unitid)
	end
end

function GridStatusIncomingHealth:Grid_UnitChanged(name, unitid)
	self:Update(unitid)
end

function GridStatusIncomingHealth:Update(unitid)
    if not unitid then
        return
    end
    if string.find(unitid, "pet") then
        return
    end

    local name, realm = UnitName(unitid)
    local IHName = name

    if realm then
        IHName = name.."-"..realm
    end

    local incoming = IHL:GetTargetIncomingHealInfo(IHName)
    local incoming = incoming + IHL:GetTargetRunningHotInfo(IHName)

    --DEFAULT_CHAT_FRAME:AddMessage("IH i "..incoming);

    if not incoming or incoming <= 0 then
        incoming = 0
        --self.core:SendStatusLost(name, "unitIncomingHealth")
        --return
    end

    local cur, max = UnitHealth(unitid), UnitHealthMax(unitid)

    if not cur or not max or max <= 0 then
        --DEFAULT_CHAT_FRAME:AddMessage("error no cur or max");
        return
    end


    local curIH = cur + incoming
    if curIH > max then
        curIH = max
    end

    local amount = string.format("%.1f", max / 1000)

    local settings = self.db.profile.unitIncomingHealth

    self.core:SendStatusGained(name, "unitIncomingHealth",
				    settings.priority,
				    (settings.range and 40),
				    settings.color,
					amount,
					curIH, max)
end

function GridStatusIncomingHealth:UpdateHealth(unitid, ignoreRange)
	self:Update(unitid)
end

function GridStatusIncomingHealth:UpdateIncoming(target, source)
    local _,_,name = string.find(target, "(.+)%-+.*")
    if not name then
        name = target
    end
    local id = RL:GetUnitIDFromName(name)
	self:Update(id);
end


